<?php
/***************************************************************************
 *                               functions.php
 *                            -------------------
 *   begin                : Saturday, Feb 13, 2001
 *   copyright            : (C) 2001 The phpBB Group
 *   email                : support@phpbb.com
 *
 *   $Id: functions.php,v 1.133.2.34 2005/02/21 18:37:33 acydburn Exp $
 *
 ***************************************************************************/

/***************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 ***************************************************************************/

if ( !defined('IN_PHPBB') )
{
	die('Hacking attempt');
}

/*
* extract_current_page
* function taken from phpBB3 - Olympus
* @param string $root_path current root path (phpbb_root_path)
*/
function extract_current_page($root_path)
{
	$page_array = array();

	// First of all, get the request uri...
	$script_name = ( !empty($_SERVER['PHP_SELF']) ) ? $_SERVER['PHP_SELF'] : getenv('PHP_SELF');
	$args = ( !empty($_SERVER['QUERY_STRING']) ) ? explode('&', $_SERVER['QUERY_STRING']) : explode('&', getenv('QUERY_STRING'));

	// If we are unable to get the script name we use REQUEST_URI as a failover and note it within the page array for easier support...
	if (!$script_name)
	{
		$script_name = ( !empty($_SERVER['REQUEST_URI']) ) ? $_SERVER['REQUEST_URI'] : getenv('REQUEST_URI');
		$page_array['failover'] = 1;
	}

	// Replace backslashes and doubled slashes (could happen on some proxy setups)
	$script_name = str_replace(array('\\', '//'), '/', $script_name);

	// Now, remove the sid and let us get a clean query string...
	foreach ($args as $key => $argument)
	{
		if (strpos($argument, 'sid=') === 0)
		{
			unset($args[$key]);
			break;
		}
	}

	// The following examples given are for an request uri of {path to the phpbb directory}/adm/index.php?i=10&b=2

	// The current query string
	$query_string = trim(implode('&', $args));

	// basenamed page name (for example: index.php)
	$page_name = htmlspecialchars(basename($script_name));

	// current directory within the phpBB root (for example: adm)
	$root_dirs = explode('/', str_replace('\\', '/', realpath($root_path)));
	$page_dirs = explode('/', str_replace('\\', '/', realpath('./')));
	$intersection = array_intersect_assoc($root_dirs, $page_dirs);

	$root_dirs = array_diff_assoc($root_dirs, $intersection);
	$page_dirs = array_diff_assoc($page_dirs, $intersection);

	$page_dir = str_repeat('../', sizeof($root_dirs)) . implode('/', $page_dirs);

	if ($page_dir && substr($page_dir, -1, 1) == '/')
	{
		$page_dir = substr($page_dir, 0, -1);
	}

	// Current page from phpBB root (for example: adm/index.php?i=10&b=2)
	$page = (($page_dir) ? $page_dir . '/' : '') . $page_name . (($query_string) ? '?' . $query_string : '');

	// The script path from the webroot to the current directory (for example: /phpBB2/adm/) : always prefixed with / and ends in /
	$script_path = trim(str_replace('\\', '/', dirname($script_name)));

	// The script path from the webroot to the phpBB root (for example: /phpBB2/)
	$script_dirs = explode('/', $script_path);
	array_splice($script_dirs, -sizeof($page_dirs));
	$root_script_path = implode('/', $script_dirs) . (sizeof($root_dirs) ? '/' . implode('/', $root_dirs) : '');

	// We are on the base level (phpBB root == webroot), lets adjust the variables abit...
	if (!$root_script_path)
	{
		$root_script_path = ($page_dir) ? str_replace($page_dir, '', $script_path) : $script_path;
	}

	$script_path .= (substr($script_path, -1, 1) == '/') ? '' : '/';
	$root_script_path .= (substr($root_script_path, -1, 1) == '/') ? '' : '/';

	$page_array += array(
		'root_script_path'	=> htmlspecialchars($root_script_path),
		'script_path'				=> htmlspecialchars($script_path),
		'page'							=> $page,
		'page_dir'					=> $page_dir,
		'page_name'					=> $page_name,
		'query_string'			=> $query_string,
		'page_full'					=> $page_name . (($query_string) ? '?' . $query_string : ''),
	);

	return $page_array;
}

/**
* Set variable, used by {@link request_var the request_var function}
* function taken from phpBB3 - Olympus
* @access private
*/
function set_var(&$result, $var, $type, $multibyte = false)
{
	settype($var, $type);
	$result = $var;

	if ($type == 'string')
	{
		$result = trim(htmlspecialchars(str_replace(array("\r\n", "\r"), array("\n", "\n"), $result), ENT_COMPAT, 'UTF-8'));

		if (!empty($result))
		{
			// Make sure multibyte characters are wellformed
			if ($multibyte)
			{
				if (!preg_match('/^./u', $result))
				{
					$result = '';
				}
			}
			else
			{
				// no multibyte, allow only ASCII (0-127)
				$result = preg_replace('/[\x80-\xFF]/', '?', $result);
			}
		}

		$result = (STRIP) ? stripslashes($result) : $result;
	}
}

/**
* Used to get passed variable
* function taken from phpBB3 - Olympus
*/
function request_var($var_name, $default, $multibyte = false, $cookie = false)
{
	if (!$cookie && isset($_COOKIE[$var_name]))
	{
		if (!isset($_GET[$var_name]) && !isset($_POST[$var_name]))
		{
			return (is_array($default)) ? array() : $default;
		}
		$_REQUEST[$var_name] = isset($_POST[$var_name]) ? $_POST[$var_name] : $_GET[$var_name];
	}

	if (!isset($_REQUEST[$var_name]) || (is_array($_REQUEST[$var_name]) && !is_array($default)) || (is_array($default) && !is_array($_REQUEST[$var_name])))
	{
		return (is_array($default)) ? array() : $default;
	}

	$var = $_REQUEST[$var_name];
	if (!is_array($default))
	{
		$type = gettype($default);
	}
	else
	{
		list($key_type, $type) = each($default);
		$type = gettype($type);
		$key_type = gettype($key_type);
		if ($type == 'array')
		{
			reset($default);
			$default = current($default);
			list($sub_key_type, $sub_type) = each($default);
			$sub_type = gettype($sub_type);
			$sub_type = ($sub_type == 'array') ? 'NULL' : $sub_type;
			$sub_key_type = gettype($sub_key_type);
		}
	}

	if (is_array($var))
	{
		$_var = $var;
		$var = array();

		foreach ($_var as $k => $v)
		{
			set_var($k, $k, $key_type);
			if ($type == 'array' && is_array($v))
			{
				foreach ($v as $_k => $_v)
				{
					if (is_array($_v))
					{
						$_v = null;
					}
					set_var($_k, $_k, $sub_key_type);
					set_var($var[$k][$_k], $_v, $sub_type, $multibyte);
				}
			}
			else
			{
				if ($type == 'array' || is_array($v))
				{
					$v = null;
				}
				set_var($var[$k], $v, $type, $multibyte);
			}
		}
	}
	else
	{
		set_var($var, $var, $type, $multibyte);
	}

	return $var;
}

/**
* Used to get multiple passed variable and create an output string
*/
function mg_get_http_vars($vars_array)
{
	//$vars_array = array('var1', 'var2');
	$http_vars = array();
	$http_vars['full_string'] = '';

	if( is_array($vars_array) )
	{
		while( list($k, $v) = each($vars_array) )
		{
			$http_vars[$v] = !empty($_GET[$v]) ? $_GET[$v] : (!empty($_POST[$v]) ? $_POST[$v] : false);
			if ($http_vars[$v])
			{
				$http_vars['full_string'] .= '&amp;' . $v . '=' . $http_vars[$v];
			}
		}
		@reset($vars_array);
	}

	return $http_vars;
}

/**
* Used to get passed var
*/
function mg_get_single_var($var)
{
	$output_var = !empty($_GET[$v]) ? $_GET[$var] : (!empty($_POST[$var]) ? $_POST[$var] : false);
	return $output_var;
}

function get_db_stat($mode)
{
	global $db, $board_config;

	switch( $mode )
	{
		case 'usercount':
			$sql = "SELECT COUNT(user_id) AS total
				FROM " . USERS_TABLE . "
				WHERE user_id <> " . ANONYMOUS;
			break;

		case 'newestuser':
			if ($board_config['inactive_users_memberlists'] == true)
			{
				$sql_active_users = '';
			}
			else
			{
				$sql_active_users = 'AND user_active = 1';
			}
			$sql = "SELECT user_id, username
				FROM " . USERS_TABLE . "
				WHERE user_id <> " . ANONYMOUS . "
				$sql_active_users
				ORDER BY user_id DESC
				LIMIT 1";
			break;
		//Battle of the Sexes disabled
		/*
		case 'gender-male':
				$sql = "SELECT COUNT(user_id) AS total_male
						FROM " . USERS_TABLE . "
						WHERE user_gender = '1'";
				break;
		case 'gender-female':
				$sql = "SELECT COUNT(user_id) AS total_female
						FROM " . USERS_TABLE . "
						WHERE user_gender = '2'";
				break;
		*/
		//End Of Battle Sexes disabled
		case 'postcount':
		case 'topiccount':
			$sql = "SELECT SUM(forum_topics) AS topic_total, SUM(forum_posts) AS post_total
				FROM " . FORUMS_TABLE;
			break;

	}

	if ( !($result = $db->sql_query($sql)) )
	{
		return false;
	}

	$row = $db->sql_fetchrow($result);

	switch ( $mode )
	{
		case 'usercount':
			return $row['total'];
			break;
		case 'newestuser':
			return $row;
			break;
		case 'postcount':
			return $row['post_total'];
			break;
		case 'topiccount':
			return $row['topic_total'];
			break;
		//Battle of the Sexes disabled
		/*
		case 'gender-male':
			return $row['total_male'];
			break;
		case 'gender-female':
			return $row['total_female'];
			break;
		*/
		//End Of Battle Sexes disabled
	}

	return false;
}

function get_gravatar($email)
{
	global $board_config;

	$image = '';
	if( !empty($email) )
	{
		$image = 'http://www.gravatar.com/avatar.php?gravatar_id=' . md5($email);

		if( $board_config['gravatar_rating'] )
		{
			$image .= '&amp;rating=' . $board_config['gravatar_rating'];
		}

		if( $board_config['gravatar_default_image'] )
		{
			$server_protocol = ($board_config['cookie_secure']) ? 'https://' : 'http://';
			$server_name = preg_replace('#^\/?(.*?)\/?$#', '\1', trim($board_config['server_name']));
			$server_port = ($board_config['server_port'] <> 80) ? ':' . trim($board_config['server_port']) : '';
			$script_name = preg_replace('#^\/?(.*?)\/?$#', '\1', trim($board_config['script_path']));
			$script_name = ($script_name == '') ? $script_name : '/' . $script_name;
			$url = preg_replace('#^\/?(.*?)\/?$#', '/\1', trim($board_config['gravatar_default_image']));

			$default_image = $server_protocol . $server_name . $server_port . $script_name . $url;
			$image .= '&amp;default=' . urlencode($default_image);
		}

		$max_size = ( $board_config['avatar_max_width'] <= $board_config['avatar_max_height'] ) ? $board_config['avatar_max_width'] : $board_config['avatar_max_height'];
		$image .= ( $max_size < 80 ) ? '&amp;size=' . $max_size : '';
	}

	return $image;
}

// added at phpBB 2.0.11 to properly format the username
function phpbb_clean_username($username)
{
	$username = substr(htmlspecialchars(str_replace("\'", "'", trim($username))), 0, 25);
	$username = phpbb_rtrim($username, "\\");
	$username = str_replace("'", "\'", $username);

	return $username;
}

/**
* This function is a wrapper for ltrim, as charlist is only supported in php >= 4.1.0
* Added in phpBB 2.0.18
*/
function phpbb_ltrim($str, $charlist = false)
{
	if ($charlist === false)
	{
		return ltrim($str);
	}

	$php_version = explode('.', PHP_VERSION);

	// php version < 4.1.0
	if ((int) $php_version[0] < 4 || ((int) $php_version[0] == 4 && (int) $php_version[1] < 1))
	{
		while ($str{0} == $charlist)
		{
			$str = substr($str, 1);
		}
	}
	else
	{
		$str = ltrim($str, $charlist);
	}

	return $str;
}

/**
* Our own generator of random values
* This uses a constantly changing value as the base for generating the values
* The board wide setting is updated once per page if this code is called
* With thanks to Anthrax101 for the inspiration on this one
* Added in phpBB 2.0.20
*/
function dss_rand()
{
	global $db, $board_config, $dss_seeded;

	$val = $board_config['rand_seed'] . microtime();
	$val = md5($val);
	$board_config['rand_seed'] = md5($board_config['rand_seed'] . $val . 'a');

	if($dss_seeded !== true)
	{
		$sql = "UPDATE " . CONFIG_TABLE . " SET
			config_value = '" . $board_config['rand_seed'] . "'
			WHERE config_name = 'rand_seed'";

		if( !$db->sql_query($sql) )
		{
			message_die(GENERAL_ERROR, "Unable to reseed PRNG", "", __LINE__, __FILE__, $sql);
		}

		$dss_seeded = true;
	}

	return substr($val, 4, 16);
}

// added at phpBB 2.0.12 to fix a bug in PHP 4.3.10 (only supporting charlist in php >= 4.1.0)
function phpbb_rtrim($str, $charlist = false)
{
	if ($charlist === false)
	{
		return rtrim($str);
	}

	$php_version = explode('.', PHP_VERSION);

	// php version < 4.1.0
	if ((int) $php_version[0] < 4 || ((int) $php_version[0] == 4 && (int) $php_version[1] < 1))
	{
		while ($str{strlen($str)-1} == $charlist)
		{
			$str = substr($str, 0, strlen($str)-1);
		}
	}
	else
	{
		$str = rtrim($str, $charlist);
	}

	return $str;
}

// Get Userdata, $user can be username or user_id. If force_str is true, the username will be forced.
function get_userdata($user, $force_str = false)
{
	global $db;

	if (!is_numeric($user) || $force_str)
	{
		$user = phpbb_clean_username($user);
	}
	else
	{
		$user = intval($user);
	}

	$sql = "SELECT *
		FROM " . USERS_TABLE . "
		WHERE ";
	$sql .= ( ( is_integer($user) ) ? "user_id = $user" : "username = '" .  str_replace("\'", "''", $user) . "'" ) . " AND user_id <> " . ANONYMOUS;
	if ( !($result = $db->sql_query($sql)) )
	{
		message_die(GENERAL_ERROR, 'Tried obtaining data for a non-existent user', '', __LINE__, __FILE__, $sql);
	}
	// Start Advanced IP Tools Pack MOD
	if ( $db->sql_affectedrows() == 0 )
	{
		message_die(GENERAL_ERROR, 'User does not exist.');
	}
	// End Advanced IP Tools Pack MOD

	return ( $row = $db->sql_fetchrow($result) ) ? $row : false;
}

function get_userdata_notifications($user, $force_str = false)
{
	global $db;

	if (!is_numeric($user) || $force_str)
	{
		$user = phpbb_clean_username($user);
	}
	else
	{
		$user = intval($user);
	}

	$sql = "SELECT *
			FROM " . USERS_TABLE . "
			WHERE ";
	$sql .= ( ( is_integer($user) ) ? "user_id = $user" : "username = '" . str_replace("\'", "''", $user) . "'" ) . " AND user_id <> " . ANONYMOUS;
	if ( !($result = $db->sql_query($sql)) )
	{
		message_die(GENERAL_ERROR, 'Tried obtaining data for a non-existent user', '', __LINE__, __FILE__, $sql);
	}
	return ( $row = $db->sql_fetchrow($result) ) ? $row : false;
}


function make_jumpbox($action, $match_forum_id = 0)
{
	global $template, $userdata, $lang, $db, $nav_links, $phpEx, $SID;

	return jumpbox($action, $match_forum_id);

//	$is_auth = auth(AUTH_VIEW, AUTH_LIST_ALL, $userdata);

	$sql = "SELECT c.cat_id, c.cat_title, c.cat_order
		FROM " . CATEGORIES_TABLE . " c, " . FORUMS_TABLE . " f
		".(($userdata['user_level'] == ADMIN)? "" : " AND c.cat_id<>'".HIDDEN_CAT."'" )."

		WHERE f.cat_id = c.cat_id
		GROUP BY c.cat_id, c.cat_title, c.cat_order
		ORDER BY c.cat_order";
	if ( !($result = $db->sql_query($sql, false, 'jumpbox_')) )
	{
		message_die(GENERAL_ERROR, "Couldn't obtain category list.", "", __LINE__, __FILE__, $sql);
	}

	$category_rows = array();
	while ( $row = $db->sql_fetchrow($result) )
	{
		$category_rows[] = $row;
	}

	if ( $total_categories = count($category_rows) )
	{
		$sql = "SELECT *
			FROM " . FORUMS_TABLE . "
			ORDER BY cat_id, forum_order";
		if ( !($result = $db->sql_query($sql, false, 'forums_')) )
		{
			message_die(GENERAL_ERROR, 'Could not obtain forums information', '', __LINE__, __FILE__, $sql);
		}

		$boxstring = '<select name="' . POST_FORUM_URL . '" onchange="if(this.options[this.selectedIndex].value != -1){ forms[\'jumpbox\'].submit() }"><option value="-1">' . $lang['Select_forum'] . '</option>';

		$forum_rows = array();
		while ( $row = $db->sql_fetchrow($result) )
		{
			$forum_rows[] = $row;
		}

		if ( $total_forums = count($forum_rows) )
		{
			for($i = 0; $i < $total_categories; $i++)
			{
				$boxstring_forums = '';
				for($j = 0; $j < $total_forums; $j++)
				{
					if ( $forum_rows[$j]['cat_id'] == $category_rows[$i]['cat_id'] && $forum_rows[$j]['auth_view'] <= AUTH_REG )
					{

//					if ( $forum_rows[$j]['cat_id'] == $category_rows[$i]['cat_id'] && $is_auth[$forum_rows[$j]['forum_id']]['auth_view'] )
//					{
						$selected = ( $forum_rows[$j]['forum_id'] == $match_forum_id ) ? 'selected="selected"' : '';
						$boxstring_forums .=  '<option value="' . $forum_rows[$j]['forum_id'] . '"' . $selected . '>' . $forum_rows[$j]['forum_name'] . '</option>';

						//
						// Add an array to $nav_links for the Mozilla navigation bar.
						// 'chapter' and 'forum' can create multiple items, therefore we are using a nested array.
						//
						$nav_links['chapter forum'][$forum_rows[$j]['forum_id']] = array (
							//SEO TOLKIT BEGIN
							//'url' => append_sid("viewforum.$phpEx?" . POST_FORUM_URL . "=" . $forum_rows[$j]['forum_id']),
							'url' => append_sid(make_url_friendly($forum_rows[$j]['forum_name']) . "-vf" . $forum_rows[$j]['forum_id'] . ".html") ,
							//SEO TOLKIT END
							'title' => $forum_rows[$j]['forum_name']
						);

					}
				}

				if ( $boxstring_forums != '' )
				{
					$boxstring .= '<option value="-1">&nbsp;</option>';
					$boxstring .= '<option value="-1">' . $category_rows[$i]['cat_title'] . '</option>';
					$boxstring .= '<option value="-1">----------------</option>';
					$boxstring .= $boxstring_forums;
				}
			}
		}

		$boxstring .= '</select>';
	}
	else
	{
		$boxstring .= '<select name="' . POST_FORUM_URL . '" onchange="if(this.options[this.selectedIndex].value != -1){ forms[\'jumpbox\'].submit() }"></select>';
	}

	// Let the jumpbox work again in sites having additional session id checks.
	/*
	if ( !empty($SID) )
	{
		$boxstring .= '<input type="hidden" name="sid" value="' . $userdata['session_id'] . '" />';
	}
	*/
	$boxstring .= '<input type="hidden" name="sid" value="' . $userdata['session_id'] . '" />';

	$template->set_filenames(array('jumpbox' => 'jumpbox.tpl'));
	$template->assign_vars(array(
		'L_GO' => $lang['Go'],
		'L_JUMP_TO' => $lang['Jump_to'],
		'L_SELECT_FORUM' => $lang['Select_forum'],

		'S_JUMPBOX_SELECT' => $boxstring,
		'S_JUMPBOX_ACTION' => append_sid($action)
		)
	);
	$template->assign_var_from_handle('JUMPBOX', 'jumpbox');

	return;
}

//
// Initialise user settings on page load
function init_userprefs($userdata)
{
	global $board_config, $theme, $images, $template, $lang, $phpEx, $phpbb_root_path, $db, $nav_links;
	global $mods, $list_yes_no, $tree;

	//	get all the mods settings
	$dir = @opendir($phpbb_root_path . 'includes/mods_settings');
	while( $file = @readdir($dir) )
	{
		if( preg_match("/^mod_.*?\." . $phpEx . "$/", $file) )
		{
			include_once($phpbb_root_path . 'includes/mods_settings/' . $file);
		}
	}
	@closedir($dir);

	if ( $userdata['user_id'] != ANONYMOUS )
	{
		if ( !empty($userdata['user_lang']))
		{
			$default_lang = phpbb_ltrim(basename(phpbb_rtrim($userdata['user_lang'])), "'");
		}

		if ( !empty($userdata['user_dateformat']) )
		{
			$board_config['default_dateformat'] = $userdata['user_dateformat'];
		}

		if ( isset($userdata['user_timezone']) )
		{
			$board_config['board_timezone'] = $userdata['user_timezone'];
		}
	}
	/*
	else
	{
		if ( isset($_GET[LANG_URL]) || isset($_POST[LANG_URL]) )
		{
			$board_config['default_lang'] = urldecode((isset($_GET[LANG_URL])) ? $_GET[LANG_URL] : $_POST[LANG_URL]);
			setcookie($board_config['cookie_name'] . '_lang', $board_config['default_lang'] , (time() + 86400), $board_config['cookie_path'], $board_config['cookie_domain'], $board_config['cookie_secure']);
		}
	}
	*/
	else
	{
		$default_lang = phpbb_ltrim(basename(phpbb_rtrim($board_config['default_lang'])), "'");
	}

	if ( !file_exists(@phpbb_realpath($phpbb_root_path . 'language/lang_' . $default_lang . '/lang_main.' . $phpEx)) )
	{
		if ( $userdata['user_id'] != ANONYMOUS )
		{
			// For logged in users, try the board default language next
			$default_lang = phpbb_ltrim(basename(phpbb_rtrim($board_config['default_lang'])), "'");
		}
		else
		{
			// For guests it means the default language is not present, try english
			// This is a long shot since it means serious errors in the setup to reach here,
			// but english is part of a new install so it's worth us trying
			$default_lang = 'english';
		}

		if ( !file_exists(@phpbb_realpath($phpbb_root_path . 'language/lang_' . $default_lang . '/lang_main.' . $phpEx)) )
		{
			message_die(CRITICAL_ERROR, 'Could not locate valid language pack');
		}
	}

	// If we've had to change the value in any way then let's write it back to the database
	// before we go any further since it means there is something wrong with it
	if ( ($userdata['user_id'] != ANONYMOUS) && ($userdata['user_lang'] !== $default_lang) )
	{
		$sql = 'UPDATE ' . USERS_TABLE . "
			SET user_lang = '" . $default_lang . "'
			WHERE user_lang = '" . $userdata['user_lang'] . "'";

		if ( !($result = $db->sql_query($sql)) )
		{
			message_die(CRITICAL_ERROR, 'Could not update user language info');
		}

		$userdata['user_lang'] = $default_lang;
	}
	elseif ( ($userdata['user_id'] === ANONYMOUS) && ($board_config['default_lang'] !== $default_lang) )
	{
		$sql = 'UPDATE ' . CONFIG_TABLE . "
			SET config_value = '" . $default_lang . "'
			WHERE config_name = 'default_lang'";

		if ( !($result = $db->sql_query($sql)) )
		{
			message_die(CRITICAL_ERROR, 'Could not update user language info');
		}
	}
	$board_config['default_lang'] = $default_lang;

	include($phpbb_root_path . 'language/lang_' . $board_config['default_lang'] . '/lang_main.' . $phpEx);
	include($phpbb_root_path . 'language/lang_' . $board_config['default_lang'] . '/lang_main_settings.' . $phpEx);
	include_once($phpbb_root_path . 'language/lang_' . $board_config['default_lang'] . '/lang_main_upi2db.' . $phpEx);
	include_once($phpbb_root_path . 'language/lang_' . $board_config['default_lang'] . '/lang_news.' . $phpEx);
	include_once($phpbb_root_path . 'language/lang_' . $board_config['default_lang'] . '/lang_main_attach.' . $phpEx);
	// CrackerTracker v5.x
	include_once($phpbb_root_path . 'language/lang_' . $board_config['default_lang'] . '/lang_main_cback_ctracker.' . $phpEx);
	// CrackerTracker v5.x
	// MG Logs - BEGIN
	if ($board_config['mg_log_actions'] == true)
	{
		include($phpbb_root_path . 'includes/log_http_cmd.' . $phpEx);
	}
	// MG Logs - END

	// MG Cash MOD For IP - BEGIN
	if ( defined('CASH_MOD') && defined('IN_CASHMOD') )
	{
		include($phpbb_root_path . 'language/lang_' . $board_config['default_lang'] . '/lang_cash.' . $phpEx);
	}
	// MG Cash MOD For IP - END

	if ( defined('IN_ADMIN') )
	{
		if( !file_exists(@phpbb_realpath($phpbb_root_path . 'language/lang_' . $board_config['default_lang'] . '/lang_admin.' . $phpEx)) )
		{
			$board_config['default_lang'] = 'english';
		}
		include($phpbb_root_path . 'language/lang_' . $board_config['default_lang'] . '/lang_admin.' . $phpEx);
		include_once($phpbb_root_path . 'language/lang_' . $board_config['default_lang'] . '/lang_admin_cback_ctracker.' . $phpEx);
		include_once($phpbb_root_path . 'language/lang_' . $board_config['default_lang'] . '/lang_admin_upi2db.' . $phpEx);
		include_once($phpbb_root_path . 'language/lang_' . $board_config['default_lang'] . '/lang_admin_attach.' . $phpEx);
		include_once($phpbb_root_path . 'language/lang_' . $board_config['default_lang'] . '/lang_kb.' . $phpEx);
		include_once($phpbb_root_path . 'language/lang_' . $board_config['default_lang'] . '/lang_jr_admin.' . $phpEx);
	}

	if (empty($tree['auth']))
	{
		get_user_tree($userdata);
	}
	include($phpbb_root_path . './includes/lang_extend_mac.' . $phpEx);

	// Set up style
	$old_default_style = $board_config['default_style'];
	if ( !$board_config['override_user_style'] )
	{
		if ( isset($_GET[STYLE_URL]) || isset($_POST[STYLE_URL]) )
		{
			$old_style = $board_config['default_style'];
			$board_config['default_style'] = urldecode((isset($_GET[STYLE_URL])) ? intval($_GET[STYLE_URL]) : intval($_POST[STYLE_URL]));
			$board_config['default_style'] = ($board_config['default_style'] == 0) ? $old_style : $board_config['default_style'];
			$style = $board_config['default_style'];
			if ( $theme = setup_style($style, $old_default_style, $old_style) )
			{
				if ( $userdata['user_id'] != ANONYMOUS )
				{
					// user logged in --> save new style ID in user profile
					$sql = "UPDATE " . USERS_TABLE . "
						SET user_style = " . $theme['themes_id'] . "
						WHERE user_id = " . $userdata['user_id'];
					if ( !$db->sql_query($sql) )
					{
						message_die(CRITICAL_ERROR, 'Error updating user style', '', __LINE__, __FILE__, $sql);
					}

					$userdata['user_style'] = $theme['themes_id'];
				}
				/*
				else
				{
					$board_config['default_style'] = $theme['themes_id'];
					setcookie($board_config['cookie_name'] . '_style', $board_config['default_style'] , (time() + 86400), $board_config['cookie_path'], $board_config['cookie_domain'], $board_config['cookie_secure']);
				}
				*/
				return;
			}
		}

		if ( ($userdata['user_id'] != ANONYMOUS) && ($userdata['user_style'] > 0) )
		{
			if ( $theme = setup_style($userdata['user_style'], $old_default_style) )
			{
				return;
			}
		}
	}

	$theme = setup_style($board_config['default_style'], $old_default_style);

	// Mozilla navigation bar - Default items that should be valid on all pages.
	// Defined here to correctly assign the Language Variables and be able to change the variables within code.
	$nav_links['top'] = array (
		'url' => append_sid(PORTAL_MG),
		'title' => $board_config['sitename']
	);
	$nav_links['forum'] = array (
		'url' => append_sid(FORUM_MG),
		'title' => sprintf($lang['Forum_Index'], $board_config['sitename'])
	);
	$nav_links['search'] = array (
		'url' => append_sid(SEARCH_MG),
		'title' => $lang['Search']
	);
	$nav_links['help'] = array (
		'url' => append_sid('faq.' . $phpEx),
		'title' => $lang['FAQ']
	);
	$nav_links['author'] = array (
		'url' => append_sid('memberlist.' . $phpEx),
		'title' => $lang['Memberlist']
	);
	// Add bookmarks to Navigation bar
	if ($userdata['session_logged_in'] && ($board_config['max_link_bookmarks'] > 0) )
	{
		$auth_sql = '';
		$is_auth_ary = auth(AUTH_READ, AUTH_LIST_ALL, $userdata);

		$ignore_forum_sql = '';
		while( list($key, $value) = each($is_auth_ary) )
		{
			if ( !$value['auth_read'] )
			{
				$ignore_forum_sql .= ( ( $ignore_forum_sql != '' ) ? ', ' : '' ) . $key;
			}
		}

		if ( $ignore_forum_sql != '' )
		{
			$auth_sql .= ( $auth_sql != '' ) ? " AND f.forum_id NOT IN ($ignore_forum_sql) " : "f.forum_id NOT IN ($ignore_forum_sql) ";
		}
		if ( $auth_sql != '' )
		{
			$sql = "SELECT t.topic_id, t.topic_title, f.forum_id
				FROM " . TOPICS_TABLE . "  t, " . BOOKMARK_TABLE . " b, " . FORUMS_TABLE . " f, " . POSTS_TABLE . " p
				WHERE t.topic_id = b.topic_id
					AND t.forum_id = f.forum_id
					AND t.topic_last_post_id = p.post_id
					AND b.user_id = " . $userdata['user_id'] . "
					AND $auth_sql
				ORDER BY p.post_time DESC
				LIMIT " . (intval($board_config['max_link_bookmarks']) + 1);
		}
		else
		{
			$sql = "SELECT t.topic_id, t.topic_title
				FROM " . TOPICS_TABLE . " t, " . BOOKMARK_TABLE . " b, " . POSTS_TABLE . " p
				WHERE t.topic_id = b.topic_id
					AND t.topic_last_post_id = p.post_id
					AND b.user_id = " . $userdata['user_id'] . "
				ORDER BY p.post_time DESC
				LIMIT " . (intval($board_config['max_link_bookmarks']) + 1);
		}
		if ( !($result = $db->sql_query($sql)) )
		{
			message_die(GENERAL_ERROR, 'Could not obtain post ids', '', __LINE__, __FILE__, $sql);
		}
		$post_rows = array();
		while ( $row = $db->sql_fetchrow($result) )
		{
			$post_rows[] = $row;
		}
		$db->sql_freeresult($result);

		if ( $total_posts = count($post_rows) )
		{
			// Define censored word matches
			$orig_word = array();
			$replacement_word = array();
			obtain_word_list($orig_word, $replacement_word);

			for($i = 0; $i < min($total_posts, $board_config['max_link_bookmarks']); $i++)
			{
				$topic_title = ( count($orig_word) ) ? preg_replace($orig_word, $replacement_word, $post_rows[$i]['topic_title']) : $post_rows[$i]['topic_title'];
				//
				// Add an array to $nav_links for the Mozilla navigation bar.
				// 'bookmarks' can create multiple items, therefore we are using a nested array.
				//
				$nav_links['bookmark'][$i] = array (
					'url' => append_sid(VIEWTOPIC_MG . '?' . POST_TOPIC_URL . '=' . $post_rows[$i]['topic_id']),
					'title' => $topic_title
				);
			}
			if ($total_posts > $board_config['max_link_bookmarks'])
			{
				$start = intval($board_config['max_link_bookmarks'] / $board_config['topics_per_page']) * $board_config['topics_per_page'];
				$nav_links['bookmark'][$i] = array (
					'url' => append_sid(SEARCH_MG . '?search_id=bookmarks&start=' . $start),
					'title' => $lang['More_bookmarks']
				);
			}
		}
	}
	return;
}

function setup_style($style, $old_default_style, $old_style = false)
{
	global $db, $board_config, $template, $images, $phpbb_root_path, $phpEx, $themes_style;

	if ( defined('CACHE_THEMES') )
	{
		include( $phpbb_root_path . './includes/def_themes.' . $phpEx );
		if ( empty($themes_style) )
		{
			cache_themes();
			@include( $phpbb_root_path . './includes/def_themes.' . $phpEx );
		}
	}

	if ( !empty($themes_style[$style]) )
	{
		$row = $themes_style[$style];
	}
	else
	{
		$sql = 'SELECT * FROM ' . THEMES_TABLE . ' WHERE themes_id = ' . (int) $style;
		if ( !($result = $db->sql_query($sql)) )
		{
			message_die(CRITICAL_ERROR, 'Could not query database for theme info');
		}

		if ( !($row = $db->sql_fetchrow($result)) )
		{
			// We are trying to setup a style which does not exist in the database
			// Try to fallback to the board default (if the user had a custom style)
			// and then any users using this style to the default if it succeeds
			if ( ($style != $board_config['default_style']) || ($old_style != false) )
			{
				if ($old_style != false)
				{
					$board_config['default_style'] = $old_style;
				}
				$sql = 'SELECT * FROM ' . THEMES_TABLE . ' WHERE themes_id = ' . (int) $board_config['default_style'];
				if ( !($result = $db->sql_query($sql)) )
				{
					message_die(CRITICAL_ERROR, 'Could not query database for theme info');
				}

				if ( $row = $db->sql_fetchrow($result) )
				{
					$db->sql_freeresult($result);
				}
				else
				{
					$style = $old_default_style;
					//message_die(CRITICAL_ERROR, "Could not get theme data for themes_id [$style]", '', __LINE__, __FILE__);
				}

				$sql = 'UPDATE ' . USERS_TABLE . '
					SET user_style = ' . (int) $board_config['default_style'] . "
					WHERE user_style = $style";
				if ( !($result = $db->sql_query($sql)) )
				{
					message_die(CRITICAL_ERROR, 'Could not update user theme info');
				}
			}
			else
			{
				message_die(CRITICAL_ERROR, "Could not get theme data for themes_id [$style]", '', __LINE__, __FILE__);
			}
		}
	}
	$template_path = 'templates/';
	$template_name = $row['template_name'];

	$template = new Template($phpbb_root_path . $template_path . $template_name);

	if ( $template )
	{
		$current_template_path = $template_path . $template_name;
		$current_template_cfg = $phpbb_root_path . $template_path . $template_name . '/' . $template_name . '.cfg';
		@include($current_template_cfg);

		if ( !defined('TEMPLATE_CONFIG') )
		{
			message_die(CRITICAL_ERROR, "Could not open $current_template_cfg", '', __LINE__, __FILE__);
		}

		$img_lang = ( file_exists(@phpbb_realpath($phpbb_root_path . $current_template_path . '/images/lang_' . $board_config['default_lang'])) ) ? $board_config['default_lang'] : 'english';

		while( list($key, $value) = @each($images) )
		{
			if ( !is_array($value) )
			{
				$images[$key] = str_replace('{LANG}', 'lang_' . $img_lang, $value);
			}
		}
	}

	return $row;
}

function check_style_exists($style_id)
{
	global $db, $board_config, $template, $images, $phpbb_root_path, $phpEx, $themes_style;

	if ( defined('CACHE_THEMES') )
	{
		include( $phpbb_root_path . './includes/def_themes.' . $phpEx );
		if ( empty($themes_style) )
		{
			cache_themes();
			@include( $phpbb_root_path . './includes/def_themes.' . $phpEx );
		}
	}

	if ( !empty($themes_style[$style_id]) )
	{
		$stile_exists = true;
	}
	else
	{
		$sql = 'SELECT * FROM ' . THEMES_TABLE . ' WHERE themes_id = ' . (int) $style;
		if ( !($result = $db->sql_query($sql)) )
		{
			message_die(CRITICAL_ERROR, 'Could not query database for theme info');
		}

		if ( !($row = $db->sql_fetchrow($result)) )
		{
			$stile_exists = false;
		}
		else
		{
			$stile_exists = true;
		}
	}

	return $stile_exists;
}

function encode_ip($dotquad_ip)
{
	$ip_sep = explode('.', $dotquad_ip);
	return sprintf('%02x%02x%02x%02x', $ip_sep[0], $ip_sep[1], $ip_sep[2], $ip_sep[3]);
}

function decode_ip($int_ip)
{
	$hexipbang = explode('.', chunk_split($int_ip, 2, '.'));
	return hexdec($hexipbang[0]). '.' . hexdec($hexipbang[1]) . '.' . hexdec($hexipbang[2]) . '.' . hexdec($hexipbang[3]);
}

//
// Create date/time from format and timezone
//
//
// Create calendar timestamp from timezone
//
function cal_date($gmepoch, $tz)
{
	global $board_config;
	return (strtotime(gmdate('M d Y H:i:s', $gmepoch + (3600 * $tz))));
}

function create_date($format, $gmepoch, $tz)
{
	global $board_config, $lang, $userdata, $pc_dateTime;
	static $translate;

	if ( empty($translate) && ($board_config['default_lang'] != 'english') )
	{
		@reset($lang['datetime']);
		while ( list($match, $replace) = @each($lang['datetime']) )
		{
			$translate[$match] = $replace;
		}
	}

	if ( !$userdata['session_logged_in'] )
	{
		$userdata['user_time_mode'] = $board_config['default_time_mode'];
		$userdata['user_dst_time_lag'] = $board_config['default_dst_time_lag'];
	}
	switch ( $userdata['user_time_mode'] )
	{
		case MANUAL_DST:
			$dst_sec = $userdata['user_dst_time_lag'] * 60;
			return ( !empty($translate) ) ? strtr(@gmdate($format, $gmepoch + (3600 * $tz) + $dst_sec), $translate) : @gmdate($format, $gmepoch + (3600 * $tz) + $dst_sec);
			break;
		case SERVER_SWITCH:
			$dst_sec = date('I', $gmepoch) * $userdata['user_dst_time_lag'] * 60;
			return ( !empty($translate) ) ? strtr(@gmdate($format, $gmepoch + (3600 * $tz) + $dst_sec), $translate) : @gmdate($format, $gmepoch + (3600 * $tz) + $dst_sec);
			break;
		default:
			return ( !empty($translate) ) ? strtr(@gmdate($format, $gmepoch + (3600 * $tz)), $translate) : @gmdate($format, $gmepoch + (3600 * $tz));
			break;
	}
}

function create_date_ex($format, $gmepoch, $tz)
{
	global $lang;
	static $today, $yesterday, $time;
	if(empty($today))
	{
		$today = array();
		$yesterday = array();
		$time = time();
	}
	$str = create_date($format, $gmepoch, $tz);
	if(empty($today[$format]))
	{
		$today[$format] = create_date($format, $time, $tz);
		$yesterday[$format] = create_date($format, $time - 86400, $tz);
	}
	if($str === $today[$format])
	{
		return $lang['Today_at'];
	}
	elseif($str === $yesterday[$format])
	{
		return $lang['Yesterday_at'];
	}
	return $str;
}

function create_date2($format, $gmepoch, $tz)
{
	$str = create_date_ex('d M Y', $gmepoch, $tz);
	$str .= ' ' . create_date('H:i', $gmepoch, $tz);
	return $str;
}

function create_date_thanks($format, $gmepoch, $tz)
{
	global $board_config, $lang;
	$date_day = create_date($format, $gmepoch, $tz);
	if ( $board_config['time_today'] < $gmepoch)
	{
		$date_day = $lang['Today_thanks'];
	}
	elseif ( $board_config['time_yesterday'] < $gmepoch)
	{
		$date_day = $lang['Yesterday_thanks'];
	}
	return $date_day;
}

function create_date_news($format, $gmepoch, $tz)
{
	global $board_config, $lang;
	$date_day = create_date($format, $gmepoch, $tz);
	if ( $board_config['time_today'] < $gmepoch)
	{
		$date_day = $lang['Today_thanks'];
	}
	elseif ( $board_config['time_yesterday'] < $gmepoch)
	{
		$date_day = $lang['Yesterday_thanks'];
	}
	return $date_day;
}

//
// Pagination routine, generates
// page number sequence
//
function get_page($num_items, $per_page, $start_item)
{
	$total_pages = ceil($num_items/$per_page);

	if ( $total_pages == 1 )
	{
		return '1';
		exit;
	}

	$on_page = floor($start_item / $per_page) + 1;
	$page_string = '';

	for($i = 0; $i < $total_pages + 1; $i++)
	{
		if( $i == $on_page )
		{
			$page_string = $i;
		}
	}
	return $page_string;
}

function generate_pagination($base_url, $num_items, $per_page, $start_item, $add_prevnext_text = true, $start = 'start')
{
	global $lang;

	$total_pages = ceil($num_items / $per_page);

	if ( $total_pages == 1 )
	{
		return '&nbsp;';
	}

	$on_page = floor($start_item / $per_page) + 1;

	$page_string = '';
	if ( $total_pages > 10 )
	{
		$init_page_max = ( $total_pages > 3 ) ? 3 : $total_pages;

		for($i = 1; $i < $init_page_max + 1; $i++)
		{
			$page_string .= ( $i == $on_page ) ? '<b>' . $i . '</b>' : '<a href="' . append_sid($base_url . '&amp;' . $start . '=' . ( ( $i - 1 ) * $per_page ) ) . '">' . $i . '</a>';
			if ( $i < $init_page_max )
			{
				$page_string .= ', ';
			}
		}

		if ( $total_pages > 3 )
		{
			if ( ($on_page > 1) && ($on_page < $total_pages) )
			{
				$page_string .= ( $on_page > 5 ) ? ' ... ' : ', ';

				$init_page_min = ( $on_page > 4 ) ? $on_page : 5;
				$init_page_max = ( $on_page < $total_pages - 4 ) ? $on_page : $total_pages - 4;

				for($i = $init_page_min - 1; $i < $init_page_max + 2; $i++)
				{
					$page_string .= ($i == $on_page) ? '<b>' . $i . '</b>' : '<a href="' . append_sid($base_url . '&amp;' . $start . '=' . ( ( $i - 1 ) * $per_page ) ) . '">' . $i . '</a>';
					if ( $i <  $init_page_max + 1 )
					{
						$page_string .= ', ';
					}
				}

				$page_string .= ( $on_page < $total_pages - 4 ) ? ' ... ' : ', ';
			}
			else
			{
				$page_string .= ' ... ';
			}

			for($i = $total_pages - 2; $i < $total_pages + 1; $i++)
			{
				$page_string .= ( $i == $on_page ) ? '<b>' . $i . '</b>'  : '<a href="' . append_sid($base_url . '&amp;' . $start . '=' . ( ( $i - 1 ) * $per_page ) ) . '">' . $i . '</a>';
				if( $i < $total_pages )
				{
					$page_string .= ', ';
				}
			}
		}
	}
	else
	{
		for($i = 1; $i < $total_pages + 1; $i++)
		{
			$page_string .= ( $i == $on_page ) ? '<b>' . $i . '</b>' : '<a href="' . append_sid($base_url . '&amp;' . $start . '=' . ( ( $i - 1 ) * $per_page ) ) . '">' . $i . '</a>';
			if ( $i < $total_pages )
			{
				$page_string .= ', ';
			}
		}
	}

	if ( $add_prevnext_text )
	{
		if ( $on_page > 1 )
		{
			$page_string = ' <a href="' . append_sid($base_url . '&amp;' . $start . '=' . ( ( $on_page - 2 ) * $per_page ) ) . '">' . $lang['Previous'] . '</a>&nbsp;&nbsp;' . $page_string;
		}

		if ( $on_page < $total_pages )
		{
			$page_string .= '&nbsp;&nbsp;<a href="' . append_sid($base_url . '&amp;' . $start . '=' . ( $on_page * $per_page ) ) . '">' . $lang['Next'] . '</a>';
		}

	}

	$page_string = ($page_string != '') ? $lang['Goto_page'] . ' ' . $page_string : '&nbsp;';

	return $page_string;
}

//
// This does exactly what preg_quote() does in PHP 4-ish
// If you just need the 1-parameter preg_quote call, then don't bother using this.
//
function phpbb_preg_quote($str, $delimiter)
{
	$text = preg_quote($str);
	$text = str_replace($delimiter, '\\' . $delimiter, $text);

	return $text;
}

//
// Obtain list of naughty words and build preg style replacement arrays for use by the
// calling script, note that the vars are passed as references this just makes it easier
// to return both sets of arrays
//
function obtain_word_list(&$orig_word, &$replacement_word)
{
	global $db;
	global $global_orig_word, $global_replacement_word;
	global $phpbb_root_path, $phpEx;
	if (isset($global_orig_word))
	{
		$orig_word			= $global_orig_word;
		$replacement_word	= $global_replacement_word;
	}
	else
	{
		if ( defined('CACHE_WORDS') )
		{
			@include($phpbb_root_path . './includes/def_words.' . $phpEx);
			if ( !isset($word_replacement) )
			{
				cache_words();
				@include($phpbb_root_path . './includes/def_words.' . $phpEx);
			}
		}
		if ( isset($word_replacement) )
		{
			$orig_word = array();
			$replacement_word = array();
			@reset($word_replacement);
			while ( list($word, $replacement) = @each($word_replacement) )
			{
				$orig_word[] = '#\b(' . str_replace('\*', '\w*?', preg_quote(stripslashes($word), '#')) . ')\b#i';
				$replacement_word[] = $replacement;
			}
		}
		else
		{
			// Define censored word matches
			$sql = "SELECT word, replacement
							FROM " . WORDS_TABLE . " ORDER BY length(word) DESC";
			if( !($result = $db->sql_query($sql, false, 'word_censor_')) )
			{
				message_die(GENERAL_ERROR, 'Could not get censored words from database', '', __LINE__, __FILE__, $sql);
			}

			if ( $row = $db->sql_fetchrow($result) )
			{
				do
				{
					$ic_word = ''; $ic_first = 0;
					$ic_chars = preg_split('//', $row['word'], -1, PREG_SPLIT_NO_EMPTY);
					foreach ($ic_chars as $char)
					{
						if (($ic_first == 1) && ($char != "*"))
						{
							$ic_word .= "_";
						}
						$ic_word .= $char; $ic_first = 1;
					}
					$ic_search = array('\*','z','s','a','b','l','i','o','p','_');
					$ic_replace = array('\w*?','(?:z|2)','(?:s|\$)','(?:a|\@)','(?:b|8|3)','(?:l|1|i|\!)','(?:i|1|l|\!)','(?:o|0)','(?:p|\?)','(?:_|\W)*');
					$orig_word[] = '#(?<=^|\W)(' . str_replace($ic_search, $ic_replace, phpbb_preg_quote($ic_word, '#')) . ')(?=\W|$)#i';
					$replacement_word[] = $row['replacement'];
				}
				while ( $row = $db->sql_fetchrow($result) );
			}
		}
		$global_orig_word = $orig_word;
		$global_replacement_word = $replacement_word;
	}

	return true;
}

//
// This is general replacement for die(), allows templated
// output in users (or default) language, etc.
//
// $msg_code can be one of these constants:
//
// GENERAL_MESSAGE : Use for any simple text message, eg. results
// of an operation, authorisation failures, etc.
//
// GENERAL ERROR : Use for any error which occurs _AFTER_ the
// common.php include and session code, ie. most errors in
// pages/functions
//
// CRITICAL_MESSAGE : Used when basic config data is available but
// a session may not exist, eg. banned users
//
// CRITICAL_ERROR : Used when config data cannot be obtained, eg
// no database connection. Should _not_ be used in 99.5% of cases
//
function message_die($msg_code, $msg_text = '', $msg_title = '', $err_line = '', $err_file = '', $sql = '')
{
	global $db, $template, $board_config, $theme, $lang, $phpEx, $phpbb_root_path, $nav_links, $gen_simple_header, $images;
	global $head_foot_ext, $cms_wide_blocks, $cms_page_id, $cms_config_vars;
	global $userdata, $user_ip, $session_length, $starttime, $tree;

	//+MOD: Fix message_die for multiple errors MOD
	static $msg_history;
	if( !isset($msg_history) )
	{
		$msg_history = array();
	}
	$msg_history[] = array(
		'msg_code' => $msg_code,
		'msg_text' => $msg_text,
		'msg_title' => $msg_title,
		'err_line' => $err_line,
		'err_file' => $err_file,
		'sql' => $sql
	);
	//-MOD: Fix message_die for multiple errors MOD
	if(defined('HAS_DIED'))
	{
	//+MOD: Fix message_die for multiple errors MOD

		//
		// This message is printed at the end of the report.
		// Of course, you can change it to suit your own needs. ;-)
		//
		$custom_error_message = 'Please, contact the %swebmaster%s. Thank you.';
		if ( !empty($board_config) && !empty($board_config['board_email']) )
		{
			$custom_error_message = sprintf($custom_error_message, '<a href="mailto:' . $board_config['board_email'] . '">', '</a>');
		}
		else
		{
			$custom_error_message = sprintf($custom_error_message, '', '');
		}
		echo "<html>\n<body>\n<b>Critical Error!</b><br />\nmessage_die() was called multiple times.<br />&nbsp;<hr />";
		for( $i = 0; $i < count($msg_history); $i++ )
		{
			echo '<b>Error #' . ($i+1) . "</b>\n<br />\n";
			if( !empty($msg_history[$i]['msg_title']) )
			{
				echo '<b>' . $msg_history[$i]['msg_title'] . "</b>\n<br />\n";
			}
			echo $msg_history[$i]['msg_text'] . "\n<br /><br />\n";
			if( !empty($msg_history[$i]['err_line']) )
			{
				echo '<b>Line :</b> ' . $msg_history[$i]['err_line'] . '<br /><b>File :</b> ' . $msg_history[$i]['err_file'] . "</b>\n<br />\n";
			}
			if( !empty($msg_history[$i]['sql']) )
			{
				echo '<b>SQL :</b> ' . $msg_history[$i]['sql'] . "\n<br />\n";
			}
			echo "&nbsp;<hr />\n";
		}
		echo $custom_error_message . '<hr /><br clear="all">';
		die("</body>\n</html>");
	//-MOD: Fix message_die for multiple errors MOD
	}

	define('HAS_DIED', 1);


	$sql_store = $sql;

	//
	// Get SQL error if we are debugging. Do this as soon as possible to prevent
	// subsequent queries from overwriting the status of sql_error()
	//
	if ( DEBUG && ( ($msg_code == GENERAL_ERROR) || ($msg_code == CRITICAL_ERROR) ) )
	{
		$sql_error = $db->sql_error();

		$debug_text = '';

		if ( $sql_error['message'] != '' )
		{
			$debug_text .= '<br /><br />SQL Error : ' . $sql_error['code'] . ' ' . $sql_error['message'];
		}

		if ( $sql_store != '' )
		{
			$debug_text .= "<br /><br />$sql_store";
		}

		if ( $err_line != '' && $err_file != '' )
		{
			$debug_text .= '<br /><br />Line : ' . $err_line . '<br />File : ' . basename($err_file);
		}
	}

	if( empty($userdata) && ( ($msg_code == GENERAL_MESSAGE) || ($msg_code == GENERAL_ERROR) ) )
	{
		// Start session management
		$userdata = session_pagestart($user_ip);
		init_userprefs($userdata);
		// End session management
	}

	//
	// If the header hasn't been output then do it
	//
	if ( !defined('HEADER_INC') && $msg_code != CRITICAL_ERROR )
	{
		if ( empty($lang) )
		{
			if( !file_exists($phpbb_root_path . 'language/lang_' . $board_config['default_lang'] . '/lang_main.' . $phpEx) )
			{
				$board_config['default_lang'] = 'english';
			}
			include($phpbb_root_path . 'language/lang_' . $board_config['default_lang'] . '/lang_main.' . $phpEx);
			include($phpbb_root_path . 'language/lang_' . $board_config['default_lang'] . '/lang_main_settings.' . $phpEx);
			include($phpbb_root_path . './includes/lang_extend_mac.' . $phpEx);
		}

		if ( empty($template) || empty($theme) )
		{
			$theme = setup_style($board_config['default_style'], $old_default_style);
		}

		// Load the Page Header
		if ( !defined('IN_ADMIN') )
		{
			include($phpbb_root_path . 'includes/page_header.' . $phpEx);
		}
		else
		{
			include($phpbb_root_path . ADM . '/page_header_admin.' . $phpEx);
		}
	}

	switch($msg_code)
	{
		case GENERAL_MESSAGE:
			if ( $msg_title == '' )
			{
				$msg_title = $lang['Information'];
			}
			break;

		case CRITICAL_MESSAGE:
			if ( $msg_title == '' )
			{
				$msg_title = $lang['Critical_Information'];
			}
			break;

		case GENERAL_ERROR:
			if ( $msg_text == '' )
			{
				$msg_text = $lang['An_error_occured'];
			}

			if ( $msg_title == '' )
			{
				$msg_title = $lang['General_Error'];
			}
			break;

		case CRITICAL_ERROR:
			//
			// Critical errors mean we cannot rely on _ANY_ DB information being
			// available so we're going to dump out a simple echo'd statement
			//
			include($phpbb_root_path . 'language/lang_english/lang_main.' . $phpEx);
			include($phpbb_root_path . 'language/lang_english/lang_main_settings.' . $phpEx);

			if ( $msg_text == '' )
			{
				$msg_text = $lang['A_critical_error'];
			}

			if ( $msg_title == '' )
			{
				$msg_title = '<b>' . $lang['Critical_Error'] . '</b>';
			}
			break;
	}

	//
	// Add on DEBUG info if we've enabled debug mode and this is an error. This
	// prevents debug info being output for general messages should DEBUG be
	// set TRUE by accident (preventing confusion for the end user!)
	//
	if ( DEBUG && ( $msg_code == GENERAL_ERROR || $msg_code == CRITICAL_ERROR ) )
	{
		if ( $debug_text != '' )
		{
			$msg_text = $msg_text . '<br /><br /><b><u>DEBUG MODE</u></b>' . $debug_text;
		}
	}

	// MG Logs - BEGIN
	//if ( ($board_config['mg_log_actions'] == true) && ( $msg_code == GENERAL_ERROR || $msg_code == CRITICAL_ERROR ) )
	if ($board_config['mg_log_actions'] == true)
	{
		mg_log('[MSG_CODE: ' . $msg_code . '] - [MSG_TITLE: ' . $msg_title . '] - [MSG_TEXT: ' . $msg_text . ']');
	}
	// MG Logs - END

	if ( $msg_code != CRITICAL_ERROR )
	{
		if ( !empty($lang[$msg_text]) )
		{
			$msg_text = $lang[$msg_text];
		}

		if ( !defined('IN_ADMIN') )
		{
			$template->set_filenames(array('message_body' => 'message_body.tpl'));
		}
		else
		{
			$template->set_filenames(array('message_body' => ADM_TPL . 'admin_message_body.tpl'));
		}

		$template->assign_vars(array(
			'MESSAGE_TITLE' => $msg_title,
			'MESSAGE_TEXT' => $msg_text
			)
		);
		$template->pparse('message_body');

		if ( !defined('IN_ADMIN') )
		{
			include($phpbb_root_path . 'includes/page_tail.' . $phpEx);
		}
		else
		{
			include($phpbb_root_path . ADM . '/page_footer_admin.' . $phpEx);
		}
	}
	else
	{
		echo "<html>\n<body>\n" . $msg_title . "\n<br /><br />\n" . $msg_text . "</body>\n</html>";
	}

	exit;
}

//
// This function is for compatibility with PHP 4.x's realpath()
// function.  In later versions of PHP, it needs to be called
// to do checks with some functions.  Older versions of PHP don't
// seem to need this, so we'll just return the original value.
// dougk_ff7 <October 5, 2002>
function phpbb_realpath($path)
{
	global $phpbb_root_path, $phpEx;

	return (!@function_exists('realpath') || !@realpath($phpbb_root_path . 'includes/functions.' . $phpEx)) ? $path : @realpath($path);
}

function redirect($url)
{
	global $db, $board_config;

	if (!empty($db))
	{
		$db->sql_close();
	}

	if (strstr(urldecode($url), "\n") || strstr(urldecode($url), "\r") || strstr(urldecode($url), ';url'))
	{
		message_die(GENERAL_ERROR, 'Tried to redirect to potentially insecure url.');
	}

	$server_protocol = ($board_config['cookie_secure']) ? 'https://' : 'http://';
	$server_name = preg_replace('#^\/?(.*?)\/?$#', '\1', trim($board_config['server_name']));
	$server_port = ($board_config['server_port'] <> 80) ? ':' . trim($board_config['server_port']) : '';
	$script_name = preg_replace('#^\/?(.*?)\/?$#', '\1', trim($board_config['script_path']));
	$script_name = ($script_name == '') ? $script_name : '/' . $script_name;
	$url = preg_replace('#^\/?(.*?)\/?$#', '/\1', trim($url));

	// Redirect via an HTML form for PITA webservers
	if (@preg_match('/Microsoft|WebSTAR|Xitami/', getenv('SERVER_SOFTWARE')))
	{
		header('Refresh: 0; URL=' . $server_protocol . $server_name . $server_port . $script_name . $url);
		echo '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"><html><head><meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1"><meta http-equiv="refresh" content="0; url=' . $server_protocol . $server_name . $server_port . $script_name . $url . '"><title>Redirect</title></head><body><div align="center">If your browser does not support meta redirection please click <a href="' . $server_protocol . $server_name . $server_port . $script_name . $url . '">HERE</a> to be redirected</div></body></html>';
		exit;
	}

	// Behave as per HTTP/1.1 spec for others
	header('Location: ' . $server_protocol . $server_name . $server_port . $script_name . $url);
	exit;
}

// Mighty Gorgon - Full Album Pack - BEGIN
//--- FLAG operation functions
function setFlag($flags, $flag)
{
	return $flags | $flag;
}
function clearFlag($flags, $flag)
{
	return ($flags & ~$flag);
}
function checkFlag($flags, $flag)
{
	return (($flags & $flag) == $flag) ? true : false;
}
// Mighty Gorgon - Full Album Pack - END

// Add function mkrealdate for Birthday MOD
// the originate php "mktime()", does not work proberly on all OS, especially when going back in time
// before year 1970 (year 0), this function "mkrealtime()", has a mutch larger valid date range,
// from 1901 - 2099. it returns a "like" UNIX timestamp divided by 86400, so
// calculation from the originate php date and mktime is easy.
// mkrealdate, returns the number of day (with sign) from 1.1.1970.

function mkrealdate($day, $month, $birth_year)
{
	// range check months
	if ($month < 1 || $month > 12)
	{
		return "error";
	}
	// range check days
	switch ($month)
	{
		case 1:
			if ($day > 31) return "error";
			break;
		case 2:
			if ($day > 29) return "error";
			$epoch = $epoch + 31;
			break;
		case 3:
			if ($day > 31) return "error";
			$epoch = $epoch + 59;
			break;
		case 4:
			if ($day > 30) return "error" ;
			$epoch = $epoch + 90;
			break;
		case 5:
			if ($day > 31) return "error";
			$epoch = $epoch + 120;
			break;
		case 6:
			if ($day > 30) return "error";
			$epoch = $epoch + 151;
			break;
		case 7:
			if ($day > 31) return "error";
			$epoch = $epoch + 181;
			break;
		case 8:
			if ($day > 31) return "error";
			$epoch = $epoch + 212;
			break;
		case 9:
			if ($day > 30) return "error";
			$epoch = $epoch + 243;
			break;
		case 10:
			if ($day > 31) return "error";
			$epoch = $epoch + 273;
			break;
		case 11:
			if ($day > 30) return "error";
			$epoch = $epoch + 304;
			break;
		case 12:
			if ($day > 31) return "error";
			$epoch = $epoch + 334;
			break;
	}
	$epoch = $epoch + $day;
	$epoch_Y = sqrt(($birth_year - 1970)*($birth_year - 1970));
	$leapyear = round((($epoch_Y + 2) / 4) - .5);
	if (($epoch_Y + 2) % 4 == 0)
	{// curent year is leapyear
		$leapyear--;
		if ($birth_year > 1970 && $month >= 3)
		{
			$epoch = $epoch + 1;
		}
		if ($birth_year < 1970 && $month < 3)
		{
			$epoch = $epoch - 1;
		}
	}
	elseif ($month == 2 && $day > 28)
	{
		return "error";//only 28 days in feb.
	}
	//year
	if ($birth_year > 1970)
	{
		$epoch = $epoch + $epoch_Y * 365 - 1 + $leapyear;
	}
	else
	{
		$epoch = $epoch - $epoch_Y * 365 - 1 - $leapyear;
	}
	return $epoch;
}

// Add function realdate for Birthday MOD
// the originate php "date()", does not work proberly on all OS, especially when going back in time
// before year 1970 (year 0), this function "realdate()", has a much larger valid date range,
// from 1901 - 2099. it returns a "like" UNIX date format (only date, related letters may be used, due to the fact that
// the given date value should already be divided by 86400 - leaving no time information left)
// a input like a UNIX timestamp divided by 86400 is expected, so
// calculation from the originate php date and mktime is easy.
// e.g. realdate ("m d Y", 3) returns the string "1 3 1970"

// UNIX users should replace this function with the below code, since this should be faster
//
function realdate($date_syntax = "Ymd", $date = 0)
{
	return create_date($date_syntax, ($date * 86400) + 1, 0);
}

/*function realdate($date_syntax="Ymd",$date=0)
{
	global $lang;
	$i = 2;
	if ($date>=0)
	{
		return create_date($date_syntax,$date*86400+1,0);
	}
	else
	{
		$year = -(date%1461);
		$days = $date + $year*1461;
		while ($days < 0)
		{
			$year--;
			$days += 365;
			if ($i++ == 3)
			{
				$i = 0;
				$days++;
			}
		}
	}
	$leap_year = ($i==0) ? true : false;
	$months_array = ($i == 0) ?
		array (0,31,60,91,121,152,182,213,244,274,305,335,366) :
		array (0,31,59,90,120,151,181,212,243,273,304,334,365);
	for ($month=1;$month<12;$month++)
	{
		if ($days<$months_array[$month]) break;
	}

	$day=$days-$months_array[$month-1]+1;
	//you may gain speed performance by remove som of the below entry's if they are not needed/used
	return strtr ($date_syntax, array(
		'a' => '',
		'A' => '',
		'\\d' => 'd',
		'd' => ($day>9) ? $day : '0'.$day,
		'\\D' => 'D',
		'D' => $lang['day_short'][($date-3)%7],
		'\\F' => 'F',
		'F' => $lang['month_long'][$month-1],
		'g' => '',
		'G' => '',
		'H' => '',
		'h' => '',
		'i' => '',
		'I' => '',
		'\\j' => 'j',
		'j' => $day,
		'\\l' => 'l',
		'l' => $lang['day_long'][($date-3)%7],
		'\\L' => 'L',
		'L' => $leap_year,
		'\\m' => 'm',
		'm' => ($month>9) ? $month : '0'.$month,
		'\\M' => 'M',
		'M' => $lang['month_short'][$month-1],
		'\\n' => 'n',
		'n' => $month,
		'O' => '',
		's' => '',
		'S' => '',
		'\\t' => 't',
		't' => $months_array[$month]-$months_array[$month-1],
		'w' => '',
		'\\y' => 'y',
		'y' => ($year>29) ? $year-30 : $year+70,
		'\\Y' => 'Y',
		'Y' => $year+1970,
		'\\z' => 'z',
		'z' => $days,
		'\\W' => '',
		'W' => '') );
}*/
// End add - Birthday MOD


// Start add - Last visit MOD
function make_hours($base_time)
{
	global $lang;
	$years = floor($base_time/31536000);
	$base_time = $base_time - ($years*31536000);
	$weeks = floor($base_time/604800);
	$base_time = $base_time - ($weeks*604800);
	$days = floor($base_time/86400);
	$base_time = $base_time - ($days*86400);
	$hours = floor($base_time/3600);
	$base_time = $base_time - ($hours*3600);
	$min = floor($base_time/60);
	$sek = $base_time - ($min*60);
	if ($sek<10) $sek ='0'.$sek;
	if ($min<10) $min ='0'.$min;
	if ($hours<10) $hours ='0'.$hours;
	$result=(($years)?$years.' '.(($years==1)?$lang['Year']:$lang['Years']).', ':'').
	(($years || $weeks)?$weeks.' '.(($weeks==1)?$lang['Week']:$lang['Weeks']).', ':'').
	(($years || $weeks || $days) ? $days.' '.(($days==1)?$lang['Day']:$lang['Days']).', ':'').
	(($hours)?$hours.':':'00:').(($min)?$min.':' :'00:').$sek;
	return ($result)?$result:$lang['None'];
}
// End add - Last visit MOD
//====================================================================== |
//==== Start Invision View Profile ===================================== |
//==== v1.1.3 ========================================================== |
//====
function get_forum_most_active($user)
{
	global $db, $userdata;

	if ( intval($user) == 0 )
	{
		$user = trim(htmlspecialchars($user));
		$user = substr(str_replace("\\'", "'", $user), 0, 25);
		$user = str_replace("'", "\\'", $user);
	}
	else
	{
		$user = intval($user);
	}

	$sql_forum = "SELECT forum_id, forum_name FROM " . FORUMS_TABLE . "
		ORDER BY forum_id";
	if ( !($result = $db->sql_query($sql_forum)) )
	{
		message_die(GENERAL_ERROR, 'Could not obtain forums list', '', __LINE__, __FILE__, $sql_forum);
	}

	$most_active_id = array();
	while ( $line = $db->sql_fetchrow($result) )
	{
		$most_active_id[] = $line['forum_id'];
		$most_active_name[$line['forum_id']] = $line['forum_name'];
	}
	$db->sql_freeresult($result);

	$count_most_active_id = count($most_active_id);

	$most_active_posts = 0;
	$num_result = 0;

	foreach ( $most_active_id as $i )
	{
		$is_auth = auth(AUTH_VIEW, $i, $userdata);
		if ( $is_auth['auth_view'] == 1 )
		{
			$sql_most = "SELECT *
				FROM " . POSTS_TABLE . "
				WHERE forum_id = $i AND poster_id = $user";
			if ( !($result = $db->sql_query($sql_most)) )
			{
				message_die(GENERAL_ERROR, 'Tried obtaining data for a non-existent user', '', __LINE__, __FILE__, $sql_most);
			}

			if ( $db->sql_numrows($result) > $most_active_posts )
			{
				$most_active_posts = $db->sql_numrows($result);
				$most_active_foren_id = $i;
				$most_active_forum_name = $most_active_name[$i];
			}
		}
	}

	return array('forum_id' => $most_active_foren_id, 'forum_name' => $most_active_forum_name, 'posts' => $most_active_posts);
}
//====
//==== Author: Disturbed One [http://anthonycoy.com] =================== |
//==== End Invision View Profile ======================================= |
//====================================================================== |

// Limit Image Width MOD --- BEGIN
function liw_get_dimensions($image_source, $identifier = '')
{
	global $db, $board_config;

	$image_checksum = '';
	$result_rowcount = 0;

	if ( ( @extension_loaded('openssl') && strstr($image_source, 'https://') ) || strstr($image_source, 'http://') )
	{
		if ( $handle = @fopen($image_source, 'rb') )
		{
			if ( strrchr($image_source, '.') == '.gif' )
			{
				$image_checksum .= md5(@fgets($handle, 100) . $identifier);
			}
			else
			{
				for ( $line = 0; $line != 5; $line++ )
				{
					$image_checksum .= md5(@fgets($handle, 100) . $identifier);
				}
			}

			$image_checksum = md5($image_checksum);

			@fclose($handle);
		}
	}

	if ( $image_checksum )
	{
		$sql = "SELECT image_width, image_height FROM " . LIW_CACHE_TABLE . " WHERE image_checksum = '" . $image_checksum . "'";

		if ( $result = $db->sql_query($sql) )
		{
			$result_rowcount = $db->sql_numrows();

			if ( $result_rowcount > 0 )
			{
				$image_data = $db->sql_fetchrow();
			}
		}
	}

	$return = array();

	if ( !$handle )
	{
		$return[] = 1;
		$return[] = 1;
	}
	else
	{
		if ( $result_rowcount == 0 )
		{
			if ( strstr($image_source, $board_config['server_name'] . $board_config['script_path']) )
			{
				$image_source = substr($image_source, ( strpos($image_source, $board_config['server_name'] . $board_config['script_path']) + strlen($board_config['server_name'] . $board_config['script_path']) ));
				$image_source = realpath('.') . '/' . $image_source;
			}

			list($image_width, $image_height) = @getimagesize($image_source);

			if ( !empty($image_checksum) )
			{
				$sql = "INSERT INTO " . LIW_CACHE_TABLE . " (image_checksum, image_width, image_height) VALUES ('" . $image_checksum . "', '" . $image_width . "', '" . $image_height . "')";
				$db->sql_query($sql);
			}
		}
		else
		{
			$image_width = $image_data['image_width'];
			$image_height = $image_data['image_height'];
		}

		$return[] = $image_width;
		$return[] = $image_height;
	}

	return $return;
}

function generate_liw_img_popup($image_source, $image_width = '', $image_height = '', $max_image_width)
{
	global $lang;

	$rand = rand(1, 10000);
	$return = '<a name="img_' . $rand . '"><a href="#img_' . $rand . '" onClick="img_popup(\'' . str_replace("'", "\'", $image_source) . '\', ' . ( ( !empty($image_width) ) ? $image_width : '\'\'' ) . ', ' . ( ( !empty($image_height) ) ? $image_height : '\'\'' ) . ', ' . $rand  . ');"><img src="' . $image_source . '"' . ( ( !empty($image_width) ) ? ' width="' . $max_image_width . '"' : '' ) . ' alt="' . $lang['LIW_click_image'] . '" border="0"></a></a><br /><span class="gensmall">' . $lang['LIW_click_image_explain'] . '</span>';

	return $return;
}
// Limit Image Width MOD --- END

// Top X Posters
function top_posters($user_limit, $show_admin, $show_mod)
{
	global $db;
	if ( ($show_admin == true) && ($show_mod == true) )
	{
		$sql_level = "";
	}
	elseif ($show_admin == true)
	{
		$sql_level = "AND u.user_level <= 1";
	}
	elseif ($show_mod == true)
	{
		$sql_level = "AND (u.user_level = 0 OR u.user_level = 2)";
	}
	else
	{
		$sql_level = "AND u.user_level = 0";
	}
	$sql = "SELECT u.username, u.user_id, u.user_posts, u.user_level
	FROM " . USERS_TABLE . " u
	WHERE (u.user_id <> " . ANONYMOUS . ")
	" . $sql_level . "
	ORDER BY u.user_posts DESC
	LIMIT " . $user_limit;
	if ( !($result = $db->sql_query($sql, false, 'top_posters_')) )
	{
		message_die(GENERAL_ERROR, 'Could not query forum top poster information', '', __LINE__, __FILE__, $SQL);
	}
	$top_posters = '';
	while( $row = $db->sql_fetchrow($result) )
	{
		$top_posters .= ' ' . colorize_username($row['user_id']) . '(' . $row['user_posts'] . ') ';
	}
	return $top_posters;
}

function resize_avatar($avatar_url)
{
	global $board_config;
	$avatar_width = 80;
	$avatar_height = 80;
	/*
	$avatar_width = $board_config['avatar_max_width'];
	$avatar_height = $board_config['avatar_max_height'];
	if (function_exists('getimagesize'))
	{
		$pic_size = @getimagesize($avatar_url);
		if ($pic_size != false)
		{
			$pic_width = $pic_size[0];
			$pic_height = $pic_size[1];
			if ( ($pic_width < $avatar_width) && ($pic_height < $avatar_height) )
			{
				$avatar_width = $pic_width;
				$avatar_height = $pic_height;
			}
			elseif ($pic_width > $pic_height)
			{
				$avatar_height = $avatar_width * ($pic_height/$pic_width);
			}
			else
			{
				$avatar_width = $avatar_height * ($pic_width/$pic_height);
			}
		}
	}
	*/
	return ( $board_config['allow_avatar_remote'] ) ? '<img src="' . $avatar_url . '" width="' . $avatar_width . '" height="' . $avatar_height . '" alt="" />' : '';
}
//
// END MOD

// Start Autolinks For phpBB Mod
//
// Obtain list of autolink words and build preg style replacement arrays for use by the
// calling script, note that the vars are passed as references this just makes it easier
// to return both sets of arrays
//
// This is a copy of phpBB's obtain_word_list() function with slight changes.
//
function obtain_autolink_list(&$orig_autolink, &$replacement_autolink, $id)
{
	global $db;

	//$where = ( $id ) ? ' WHERE link_forum = 0 OR link_forum = ' . $id : ' WHERE link_forum = -1';
	$where = ( $id ) ? ' WHERE link_forum = 0 OR link_forum IN (' . $id . ')' : ' WHERE link_forum = -1';

	$sql = "SELECT * FROM  " . AUTOLINKS . $where;
	if( !($result = $db->sql_query($sql, false, 'autolinks_')) )
	{
		message_die(GENERAL_ERROR, 'Could not get autolink data from database', '', __LINE__, __FILE__, $sql);
	}

	if( $row = $db->sql_fetchrow($result) )
	{
		do
		{
			// Munge word boundaries to stop autolinks from linking to
			// themselves or other autolinks in step 2 in the function below.
			$row['link_url'] = preg_replace('/(\b)/', '\\1ALSPACEHOLDER', $row['link_url']);
			$row['link_comment'] = preg_replace('/(\b)/', '\\1ALSPACEHOLDER', $row['link_comment']);

			if( $row['link_style'] )
			{
				$row['link_style'] = preg_replace('/(\b)/', '\\1ALSPACEHOLDER', $row['link_style']);
				$style = ' style="' . htmlspecialchars($row['link_style']) . '" ';
			}
			else
			{
				$style = ' ';
			}
			$orig_autolink[] = '/(?<![\/\w@\.:-])(?!\.\w)(' . phpbb_preg_quote($row['link_keyword'], '/'). ')(?![\/\w@:-])(?!\.\w)/i';
			if( $row['link_int'] )
			{
				$replacement_autolink[] = '<a href="' . append_sid(htmlspecialchars($row['link_url'])) . '" target="_self"' . $style . 'title="' . htmlspecialchars($row['link_comment']) . '">' . htmlspecialchars($row['link_title']) . '</a>';
			}
			else
			{
				$replacement_autolink[] = '<a href="' . htmlspecialchars($row['link_url']) . '" target="_blank"' . $style . 'title="' . htmlspecialchars($row['link_comment']) . '">' . htmlspecialchars($row['link_title']) . '</a>';
			}
		}
		while( $row = $db->sql_fetchrow($result) );
	}

	return true;
}

//
// Taken from the POST-NUKE pnuserapi.php Autolinks user API file with slight changes.
// Original Author - Jim McDonald.
//
function autolink_transform($message, $orig, $replacement)
{
	global $board_config;

	// Step 1 - move all tags out of the text and replace them with placeholders
	preg_match_all('/(<a\s+.*?\/a>|<[^>]+>)/i', $message, $matches);
	$matchnum = count($matches[1]);
	for($i = 0; $i < $matchnum; $i++)
	{
		$message = preg_replace('/' . preg_quote($matches[1][$i], '/') . '/', "ALPLACEHOLDER{$i}PH", $message, 1);
	}

	// Step 2 - s/r of the remaining text
	if( $board_config['autolink_first'] )
	{
		$message = preg_replace($orig, $replacement, $message, 1);
	}
	else
	{
		$message = preg_replace($orig, $replacement, $message);
	}

	// Step 3 - replace the spaces we munged in step 1
	$message = preg_replace('/ALSPACEHOLDER/', '', $message);

	// Step 4 - replace the HTML tags that we removed in step 1
	for($i = 0; $i <$matchnum; $i++)
	{
		$message = preg_replace("/ALPLACEHOLDER{$i}PH/", $matches[1][$i], $message, 1);
	}

	return $message;
}
// End Autolinks For phpBB Mod

/* functions_separate.php - BEGIN */
//
// Select topic to be suggested
//
function get_dividers($topics)
{
	global $lang, $board_config;
	$dividers = array();
	$total_topics = count($topics);
	$total_by_type = array (POST_GLOBAL_ANNOUNCE => 0, POST_ANNOUNCE => 0, POST_STICKY => 0, POST_NORMAL => 0);

	for ( $i=0; $i < $total_topics; $i++ )
	{
		$total_by_type[$topics[$i]['topic_type']]++;
	}

	//$board_config['split_ga_ann_sticky'] = 2;
	$split_options = $board_config['split_ga_ann_sticky'];

	//split_ga_ann_sticky == 0 -> No split
	//split_ga_ann_sticky == 1 -> Global Announce Announce and Sticky Toghether (not splitted)
	//split_ga_ann_sticky == 2 -> Split global Announce, Announce and Sticky Toghether (splitted)
	//split_ga_ann_sticky == 3 -> All Splitted

	if ( ( $total_by_type[POST_GLOBAL_ANNOUNCE] + $total_by_type[POST_ANNOUNCE] + $total_by_type[POST_STICKY] ) != 0 )
	{
		$count_topics = 0;

		switch ( $split_options )
		{
			case '0':
				break;

			case '1':
				$dividers[$count_topics] = $lang['Announcements_and_Sticky'];
				$count_topics += $total_by_type[POST_ANNOUNCE] + $total_by_type[POST_STICKY] + $total_by_type[POST_GLOBAL_ANNOUNCE];
				break;

			case '2':
				$dividers[$count_topics] = $lang['Global_Announcements'];
				$count_topics += $total_by_type[POST_GLOBAL_ANNOUNCE];

				$dividers[$count_topics] = $lang['Announcements_and_Sticky'];
				$count_topics += $total_by_type[POST_ANNOUNCE] + $total_by_type[POST_STICKY];
				break;

			case '3':
				$dividers[$count_topics] = $lang['Global_Announcements'];
				$count_topics += $total_by_type[POST_GLOBAL_ANNOUNCE];

				$dividers[$count_topics] = $lang['Announcements'];
				$count_topics += $total_by_type[POST_ANNOUNCE];

				$dividers[$count_topics] = $lang['Sticky_Topics'];
				$count_topics += $total_by_type[POST_STICKY];

				break;

		}//end of switch

		if ( $count_topics < $total_topics )
		{
			$dividers[$count_topics] = $lang['Topics'];
		}
	}
	return $dividers;
}
/* functions_separate.php - END */

/* functions_ftr.php - END */
function GetUsersView($user)
{
		global $db, $table_prefix;
		$q1 = "SELECT *
				FROM ". FORCE_READ_USERS_TABLE . "
					WHERE user = '$user'";
		$r1 = $db -> sql_query($q1);
		$row1 = $db -> sql_fetchrow($r1);
		$user = $row1['user'];
		$read = $row1['read'];

		if(($user) && ($read == "1"))
		{
			$viewed = "true";
		}
		else
		{
			$viewed = "false";
		}

	return $viewed;
}

function InsertReadTopic($user)
{
	global $db, $table_prefix;
	$time = time();

	$q = "INSERT INTO ". FORCE_READ_USERS_TABLE . "
			VALUES
			('$user', '1', '$time')";
	$r = $db -> sql_query($q);

	return;
}
/* functions_ftr.php - END */

/* functions_bookmark.php - BEGIN */
// Checks whether a bookmark is set or not
function is_bookmark_set($topic_id)
{
	global $db, $userdata;

	$user_id = $userdata['user_id'];
	$sql = "SELECT topic_id, user_id
		FROM " . BOOKMARK_TABLE . "
		WHERE topic_id = '" . $topic_id . "'
			AND user_id = '" . $user_id . "'
		LIMIT 1";
	if ( $result = $db->sql_query($sql) )
	{
		$is_bookmark_set = ($db->sql_fetchrow($result)) ? (true) : (false);
	}
	else
	{
		message_die(GENERAL_ERROR, 'Could not obtain bookmark information', '', __LINE__, __FILE__, $sql);
		$is_bookmark_set = false;
	}
	$db->sql_freeresult($result);

	return $is_bookmark_set;
}

// Sets a bookmark
function set_bookmark($topic_id)
{
	global $db, $userdata;

	$user_id = $userdata['user_id'];
	if ( !is_bookmark_set($topic_id, $user_id) )
	{
		$sql = "INSERT INTO " . BOOKMARK_TABLE . " (topic_id, user_id)
			VALUES ($topic_id, $user_id)";
		if ( !$db->sql_query($sql) )
		{
			message_die(GENERAL_ERROR, 'Could not insert bookmark information', '', __LINE__, __FILE__, $sql);
		}
	}
	return;
}

// Removes a bookmark
function remove_bookmark($topic_id)
{
	global $db, $userdata;

	$user_id = $userdata['user_id'];
	$sql = "DELETE FROM " . BOOKMARK_TABLE . "
		WHERE topic_id IN ($topic_id) AND user_id = $user_id";
	if ( !$db->sql_query($sql) )
	{
		message_die(GENERAL_ERROR, 'Could not remove bookmark information', '', __LINE__, __FILE__, $sql);
	}
	return;
}
/* functions_bookmark.php - END */

// Make URL Friendly rewritten by MG
function make_url_friendly($url)
{

	$url = strtolower($url);

	$find = array(
		' ', '<b>', '</b>',
		'&quot;', '&amp;', '&lt;', '&gt;', '\r\n', '\n', '/', '\\', '+', '<', '>', 'vfr',
		'', '', '', '', '', '', '', '', '', '', '', '',
		'', '', '', '', '', '', '', '',
		'', '', '', '', '', '', '', '',
		'', '', '', '', '', '', '', '', '', '', '', '',
		'', '', '', '', '', '', '', '',
		'',
		'', '',
		'', '',
		'',
		'`', '', '',
		' ', '&',
	);

	$repl = array(
		'-', '', '',
		'-', '-', '-', '-', '-', '-', '-', '-', '-', '-',
		'a', 'a', 'a', 'a', 'a', 'A', 'A', 'A', 'A', 'A', 'ae', 'ae',
		'e', 'e', 'e', 'e', 'E', 'E', 'E', 'E',
		'i', 'i', 'i', 'i', 'I', 'I', 'I', 'I',
		'o', 'o', 'o', 'o', 'o', 'O', 'O', 'O', 'O', 'O', 'oe', 'oe',
		'u', 'u', 'u', 'U', 'U', 'U', 'ue', 'ue',
		'y',
		'c', 'C',
		'n', 'N',
		'ss',
		'-', '-', '-',
		'-', '-',
	);

	$url = str_replace($find, $repl, $url);

	$find = array(
		'/[^a-z0-9\-<>]/', '/[\-]+/', '/<[^>]*>/',
	);

	$repl = array(
		'', '-', '',
	);

	$url = preg_replace($find, $repl, $url);

	$url = str_replace('--', '-', $url);

	return $url;
}

// FUNCTIONS
function rewrite_urls($content)
{
	function if_query($amp)
	{
		if($amp != '')
		{
			return '?';
		}
	}

	$url_in = array(
		//'/(?<!\/)' . PORTAL_MG . '\?topic_id=([0-9]+)((&amp;)|(&)){0,1}([^>]+>)(.*?)<\/a>/e',
		'/(?<!\/)' . FORUM_MG . '\?' . POST_CAT_URL . '=([0-9]+)((&amp;)|(&)){0,1}([^>]+>)(.*?)<\/a>/e',
		'/(?<!\/)' . VIEWFORUM_MG . '\?' . POST_FORUM_URL . '=([0-9]+)((&amp;)|(&)){0,1}([^>]+>)(.*?)<\/a>/e',
		'/(?<!\/)\.\/' . VIEWFORUM_MG . '\?' . POST_FORUM_URL . '=([0-9]+)((&amp;)|(&)){0,1}([^>]+>)(.*?)<\/a>/e',
		'/(?<!\/)' . VIEWTOPIC_MG . '\?' . POST_POST_URL . '=([0-9]+)((&amp;)|(&)){0,1}([^>]+>)(.*?)<\/a>/e',
		'/(?<!\/)' . VIEWTOPIC_MG . '\?' . POST_TOPIC_URL . '=([0-9]+)((&amp;)|(&)){0,1}([^>]+>)(.*?)<\/a>/e',
		/*
		'/(?<!\/)album_cat.php\?cat_id=([0-9]+)((&amp;)|(&)){0,1}([^>]+>)(.*?)<\/a>/e',
		'/(?<!\/)album_showpage.php\?pic_id=([0-9]+)((&amp;)|(&)){0,1}([^>]+>)(<img .*?)<\/a>/e',
		'/(?<!\/)album_showpage.php\?full=&pic_id=([0-9]+)((&amp;)|(&)){0,1}([^>]+>)(<img .*?)<\/a>/e',
		'/(?<!\/)album_showpage.php\?pic_id=([0-9]+)((&amp;)|(&)){0,1}([^>]+>)(.*?)<\/a>/e',
		'/(?<!\/)album_personal.php\?user_id=([0-9]+)((&amp;)|(&)){0,1}([^>]+>)(.*?)<\/a>/e',
		'/(?<!\/)album_thumbnail.php\?pic_id=([0-9]+)((&amp;)|(&)){0,1}([^>]+>)(.*?)<\/a>/e',
		'/(?<!\/)album_pic.php\?pic_id=([0-9]+)((&amp;)|(&)){0,1}([^>]+>)(.*?)<\/a>/e',
		'/(?<!\/)album_picm.php\?pic_id=([0-9]+)((&amp;)|(&)){0,1}([^>]+>)(.*?)<\/a>/e',
		*/
		'/(?<!\/)dload.php\?action=category&cat_id=([0-9]+)((&amp;)|(&)){0,1}([^>]+>)(.*?)<\/a>/e',
		'/(?<!\/)dload.php\?action=category&cat_id=([0-9]+)((&amp;)|(&)){0,1}([^>]+>)(.*?)<\/a>/e',
		'/(?<!\/)dload.php\?action=file&file_id=([0-9]+)((&amp;)|(&)){0,1}([^>]+>)(.*?)<\/a>/e',
		'/(?<!\/)kb.php\?mode=cat&amp;cat=([0-9]+)((&amp;)|(&)){0,1}([^>]+>)(.*?)<\/a>/e',
		'/(?<!\/)kb.php\?mode=article&amp;k=([0-9]+)((&amp;)|(&)){0,1}([^>]+>)(.*?)<\/a>/e',
		'/(?<!\/)kb.php\?mode=stats&amp;stats=mostpopular((&amp;)|(&)){0,1}([^>]+>)(.*?)<\/a>/e',
		'/(?<!\/)kb.php\?mode=stats&amp;stats=toprated((&amp;)|(&)){0,1}([^>]+>)(.*?)<\/a>/e',
		'/(?<!\/)kb.php\?mode=stats&amp;stats=latest((&amp;)|(&)){0,1}([^>]+>)(.*?)<\/a>/e',
	);

	$url_out = array(
		//"make_url_friendly('\\6') . '-na\\1.html' . if_query('\\2') . stripslashes('\\5\\6') . '</a>'",
		"make_url_friendly('\\6') . '-vc\\1.html' . if_query('\\2') . stripslashes('\\5\\6') . '</a>'",
		"make_url_friendly('\\6') . '-vf\\1.html' . if_query('\\2') . stripslashes('\\5\\6') . '</a>'",
		"make_url_friendly('\\6') . '-vf\\1.html' . if_query('\\2') . stripslashes('\\5\\6') . '</a>'",
		"make_url_friendly('\\6') . '-vp\\1.html' . if_query('\\2') . stripslashes('\\5\\6') . '</a>'",
		"make_url_friendly('\\6') . '-vt\\1.html' . if_query('\\2') . stripslashes('\\5\\6') . '</a>'",
		/*
		"make_url_friendly('\\6') . '-ac\\1.html' . if_query('\\2') . stripslashes('\\5\\6') . '</a>'",
		"'show-pic-asp\\1.html' . if_query('\\2') . stripslashes('\\5\\6') . '</a>'",
		"'show-pic-full-asp\\1.html' . if_query('\\2') . stripslashes('\\5\\6') . '</a>'",
		"make_url_friendly('\\6') . '-asp\\1.html' . if_query('\\2') . stripslashes('\\5\\6') . '</a>'",
		"make_url_friendly('\\6') . '-aper\\1.html' . if_query('\\2') . stripslashes('\\5\\6') . '</a>'",
		"make_url_friendly('\\6') . '-at\\1.html' . if_query('\\2') . stripslashes('\\5\\6') . '</a>'",
		"make_url_friendly('\\6') . '-apic\\1.html' . if_query('\\2') . stripslashes('\\5\\6') . '</a>'",
		"make_url_friendly('\\6') . '-apm\\1.html' . if_query('\\2') . stripslashes('\\5\\6') . '</a>'",
		*/
		"make_url_friendly('\\6') . '-dc\\1.html' . if_query('\\2') . stripslashes('\\5\\6') . '</a>'",
		"make_url_friendly('\\6') . '-dc\\1.html' . if_query('\\2') . stripslashes('\\5\\6') . '</a>'",
		"make_url_friendly('\\6') . '-df\\1.html' . if_query('\\2') . stripslashes('\\5\\6') . '</a>'",
		"make_url_friendly('\\6') . '-kbc\\1.html' . if_query('\\2') . stripslashes('\\5\\6') . '</a>'",
		"make_url_friendly('\\6') . '-kba\\1.html' . if_query('\\2') . stripslashes('\\5\\6') . '</a>'",
		"make_url_friendly('\\5') . '-kbsmp.html' . if_query('\\1') . stripslashes('\\4\\5') . '</a>'",
		"make_url_friendly('\\5') . '-kbstr.html' . if_query('\\1') . stripslashes('\\4\\5') . '</a>'",
		"make_url_friendly('\\5') . '-kbsl.html' . if_query('\\1') . stripslashes('\\4\\5') . '</a>'",
	);

	$content = preg_replace($url_in, $url_out, $content);

	return $content;
}

/*
MG BOTS Parsing Function
*/
function bots_parse($ip_address, $bot_color = '#888888', $browser = false)
{
	/*
	// Testing!!!
	$bot_name = 'MG';
	return $bot_name;
	*/
	$bot_name = false;
	//return $bot_name;
	$bot_color = ($bot_color == '') ? '#888888' : $bot_color;

	if ($browser != false)
	{
		//if ( (strpos($browser, 'MSIE') !== false) || (strpos($browser, 'Opera') !== false) || (strpos($browser, 'Firefox') !== false) || (strpos($browser, 'Mozilla') !== false) )
		if ( (strpos($browser, 'MSIE') !== false) || (strpos($browser, 'Opera') !== false) || (strpos($browser, 'Firefox') !== false) )
		{
			$bot_name = false;
			return $bot_name;
		}
	}

	$bot_name_ary = array(
		'adsbot' => 'AdsBot [Google]',
		'alexa' => 'Alexa',
		'alta_vista' => 'Alta Vista',
		'alltheweb' => 'AllTheWeb',
		'ask_jeeves' => 'Ask Jeeves',
		'ask_jeeves_teoma' => 'Ask Jeeves',
		'baidu' => 'Baidu [Spider]',
		'becomebot' => 'Become',
		'ebay' => 'eBay Ad',
		'edintorni' => 'eDintorni Crawler',
		'exabot' => 'Exabot',
		'fast_enterprise' => 'FAST Enterprise [Crawler]',
		'fast_webcrawler' => 'FAST WebCrawler [Crawler]',
		'francis' => 'Francis',
		'gigablast' => 'Gigablast',
		'gigabot' => 'Gigabot',
		'google_adsense' => 'Google Adsense',
		'google_desktop' => 'Google Desktop',
		'google_feedfetcher' => 'Google Feedfetcher',
		'google' => 'Google',
		'heise_it_markt' => 'Heise IT-Markt [Crawler]',
		'heritrix' => 'Heritrix [Crawler]',
		'jetbot' => 'JetBot',
		'ibm_research' => 'IBM Research',
		'iccrawler_icjobs' => 'ICCrawler - ICjobs',
		'ichiro' => 'ichiro [Crawler]',
		'ie_auto_discovery' => 'IEAutoDiscovery',
		'indy_library' => 'Indy Library',
		'infoseek' => 'Infoseek',
		'inktomi' => 'Inktomi',
		'live' => 'LiveBot',
		'looksmart' => 'LookSmart',
		'lycos' => 'Lycos',
		'magpierss' => 'MagpieRSS',
		'majestic_12' => 'Majestic-12',
		'metager' => 'Metager',
		'msn_newsblogs' => 'MSN NewsBlogs',
		'msn' => 'MSN',
		'msnbot_media' => 'MSNbot Media',
		'msrbot_media' => 'Microsoft Research',
		'ng_search' => 'NG-Search',
		'noxtrum' => 'Noxtrum [Crawler]',
		'nutch' => 'Nutch',
		'nutch_cvs' => 'Nutch/CVS',
		'omniexplorer' => 'OmniExplorer',
		'online_link' => 'Online link [Validator]',
		'perl' => 'Perl Script',
		'pompos' => 'Pompos',
		'psbot' => 'psbot [Picsearch]',
		'seekport' => 'Seekport',
		'sensis' => 'Sensis [Crawler]',
		'seo_crawler' => 'SEO Crawler [Crawler]',
		'seoma' => 'Seoma [Crawler]',
		'seosearch' => 'SEOSearch [Crawler]',
		'snapbot' => 'Snap Bot',
		'snappy' => 'Snappy',
		'speedy_spider' => 'Speedy Spider',
		'steeler' => 'Steeler [Crawler]',
		'synoo' => 'Synoo',
		'telekom' => 'Telekom',
		'turnitinbot' => 'TurnitinBot',
		'twiceler' => 'Twiceler',
		'virgilio' => 'Virgilio',
		'voila' => 'Voila',
		'voyager' => 'Voyager',
		'w3' => 'W3 [Sitesearch]',
		'w3c_linkcheck' => 'W3C [Linkcheck]',
		'w3c_validator' => 'W3C [Validator]',
		'wisenut' => 'WiseNut',
		'yacy' => 'YaCy',
		'yahoo_mmcrawler' => 'Yahoo MMCrawler',
		'yahoo_slurp' => 'Yahoo! DE Slurp',
		'yahoo' => 'Yahoo! Slurp',
		'yahooseeker' => 'YahooSeeker',
	);

	$bot_color_ary = array(
		'google' => '<span style="font-weight: bold; color: #2244BB;">G</span><span style="font-weight: bold; color: #DD2222;">o</span><span style="font-weight: bold; color: #EEBB00;">o</span><span style="font-weight: bold; color: #2244BB;">g</span><span style="font-weight: bold; color: #339933;">l</span><span style="font-weight: bold; color: #DD2222;">e</span>',
		'google_adsense' => '<span style="font-weight: bold; color: #2244BB;">G</span><span style="font-weight: bold; color: #DD2222;">o</span><span style="font-weight: bold; color: #EEBB00;">o</span><span style="font-weight: bold; color: #2244BB;">g</span><span style="font-weight: bold; color: #339933;">l</span><span style="font-weight: bold; color: #DD2222;">e</span><span style="font-weight: bold; color: #DD2222;"> Adsense</span>',
		'google_desktop' => '<span style="font-weight: bold; color: #2244BB;">G</span><span style="font-weight: bold; color: #DD2222;">o</span><span style="font-weight: bold; color: #EEBB00;">o</span><span style="font-weight: bold; color: #2244BB;">g</span><span style="font-weight: bold; color: #339933;">l</span><span style="font-weight: bold; color: #DD2222;">e</span><span style="font-weight: bold; color: #DD2222;"> Desktop</span>',
		'google_feedfetcher' => '<span style="font-weight: bold; color: #2244BB;">G</span><span style="font-weight: bold; color: #DD2222;">o</span><span style="font-weight: bold; color: #EEBB00;">o</span><span style="font-weight: bold; color: #2244BB;">g</span><span style="font-weight: bold; color: #339933;">l</span><span style="font-weight: bold; color: #DD2222;">e</span><span style="font-weight: bold; color: #DD2222;"> Feedfetcher</span>',
		'yahoo_mmcrawler' => '<span style="font-weight: bold; color: #DD2222;">Yahoo!</span><span style="font-weight: bold; color: #2244BB;"> MMCrawler</span>',
		'yahoo_slurp' => '<span style="font-weight: bold; color: #DD2222;">Yahoo!</span><span style="font-weight: bold; color: #2244BB;"> DE Slurp</span><span style="font-weight: bold; color: ' . $bot_color . ';"> [Bot]</span>',
		'yahoo' => '<span style="font-weight: bold; color: #DD2222;">Yahoo!</span><span style="font-weight: bold; color: #2244BB;"> Slurp</span>',
		'live' => '<span style="font-weight: bold; color: #446688;">LiveBot</span>',
		'msn_newsblogs' => '<span style="font-weight: bold; color: #446688;">MSN NewsBlogs</span>',
		'msn' => '<span style="font-weight: bold; color: #446688;">MSN</span>',
		'msnbot_media' => '<span style="font-weight: bold; color: #446688;">MSNbot Media</span>',
	);

	// list more probable first... to speed up things!
	$bot_reg_exp_ary = array(
		'yahoo' => 'Yahoo! Slurp',
		'google' => 'Googlebot',
		'msn' => 'msnbot/',
		'live' => 'LiveBot',
		'adsbot' => 'AdsBot-Google',
		'google_adsense' => 'Mediapartners-Google',
		'yahoo_slurp' => 'Yahoo! DE Slurp',
		'yahoo_mmcrawler' => 'Yahoo-MMCrawler/',
		'yahooseeker' => 'YahooSeeker/',
		'google_desktop' => 'Google Desktop',
		'google_feedfetcher' => 'Feedfetcher-Google',
		'msn_newsblogs' => 'msnbot-NewsBlogs/',
		'msnbot_media' => 'msnbot-media/',
		'alexa' => 'ia_archiver',
		'alta_vista' => 'Scooter/',
		'alltheweb' => 'alltheweb',
		'ask_jeeves' => 'Ask Jeeves',
		'ask_jeeves_teoma' => 'teoma',
		'baidu' => 'Baiduspider+(',
		'becomebot' => 'BecomeBot/',
		'edintorni' => 'eDintorni',
		'exabot' => 'Exabot/',
		'fast_enterprise' => 'FAST Enterprise Crawler',
		'fast_webcrawler' => 'FAST-WebCrawler/',
		'francis' => 'http://www.neomo.de/',
		'gigabot' => 'Gigabot/',
		'heise_it_markt' => 'heise-IT-Markt-Crawler',
		'heritrix' => 'heritrix/1.',
		'jetbot' => 'Jetbot',
		'ibm_research' => 'ibm.com/cs/crawler',
		'iccrawler_icjobs' => 'ICCrawler - ICjobs',
		'ichiro' => 'ichiro/2',
		'ie_auto_discovery' => 'IEAutoDiscovery',
		'indy_library' => 'Indy Library',
		'infoseek' => 'Infoseek',
		'live' => 'LiveBot',
		'looksmart' => 'MARTINI',
		'lycos' => 'Lycos',
		'magpierss' => 'MagpieRSS',
		'majestic_12' => 'MJ12bot/',
		'metager' => 'MetagerBot/',
		'msrbot_media' => 'MSRBOT',
		'ng_search' => 'NG-Search/',
		'noxtrum' => 'noxtrumbot',
		'nutch' => 'http://lucene.apache.org/nutch/',
		'nutch_cvs' => 'NutchCVS/',
		'omniexplorer' => 'OmniExplorer_Bot/',
		'online_link' => 'online link validator',
		'perl' => 'libwww-perl/',
		'psbot' => 'psbot/0',
		'seekport' => 'Seekbot/',
		'sensis' => 'Sensis Web Crawler',
		'seo_crawler' => 'SEO search Crawler/',
		'seoma' => 'Seoma',
		'seosearch' => 'SEOsearch/',
		'snapbot' => 'Snapbot/',
		'snappy' => 'Snappy/',
		'speedy_spider' => 'Speedy Spider',
		'steeler' => 'http://www.tkl.iis.u-tokyo.ac.jp/~crawler/',
		'synoo' => 'SynooBot/',
		'telekom' => 'crawleradmin.t-info@telekom.de',
		'turnitinbot' => 'TurnitinBot/',
		'twiceler' => 'Twiceler',
		'voyager' => 'voyager/1.0',
		'voila' => 'VoilaBot',
		'w3' => 'W3 SiteSearch Crawler',
		'w3c_linkcheck' => 'W3C-checklink/',
		'w3c_validator' => 'W3C_',
		'wisenut' => 'http://www.WISEnutbot.com',
		'yacy' => 'yacybot',
	);

	if ($browser != false)
	{
		while ($str_check = current($bot_reg_exp_ary))
		{
			if (strpos(strtolower($browser), strtolower($str_check)) !== false)
			{
				$bot_name = array_key_exists(key($bot_reg_exp_ary), $bot_color_ary) ? $bot_color_ary[key($bot_reg_exp_ary)] : '<span style="font-weight: bold; color: ' . $bot_color . ';">' . $bot_name_ary[key($bot_reg_exp_ary)] . '</span>';
				return $bot_name;
			}
			next($bot_reg_exp_ary);
		}
	}

	$tmp_list = explode(".", decode_ip($ip_address));

	if ($tmp_list[0] == "66" && $tmp_list[1] == "249")
	{
		return $bot_color_ary['google'];
	}
	elseif ($tmp_list[0] == "72" && $tmp_list[1] == "14" && $tmp_list[2] == "199")
	{
		return $bot_color_ary['google_feedfetcher'];
	}
	elseif ( ($tmp_list[0] == "66" && $tmp_list[1] == "196") || ($tmp_list[0] == "68" && $tmp_list[1] == "142") || ($tmp_list[0] == "72" && $tmp_list[1] == "30") || ($tmp_list[0] == "74" && $tmp_list[1] == "6") || ($tmp_list[0] == "202" && $tmp_list[1] == "160" && $tmp_list[2] == "180") )
	{
		return $bot_color_ary['yahoo'];
	}
	elseif ( ($tmp_list[0] == "207" && $tmp_list[1] == "66" && $tmp_list[2] == "146") || ($tmp_list[0] == "207" && $tmp_list[1] == "46") || ($tmp_list[0] == "65" && $tmp_list[1] == "54" && $tmp_list[2] == "188") || ($tmp_list[0] == "65" && $tmp_list[1] == "54" && $tmp_list[2] == "246") || ($tmp_list[0] == "65" && $tmp_list[1] == "55" && $tmp_list[2] == "210") || ($tmp_list[0] == "65" && $tmp_list[1] == "55" && $tmp_list[2] == "213") || ($tmp_list[0] == "65" && $tmp_list[1] == "54" && $tmp_list[2] == "165") )
	{
		return $bot_color_ary['msn'];
	}
	elseif ($tmp_list[0] == "195" && $tmp_list[1] == "101" && $tmp_list[2] == "94")
	{
		$bot_name = '<span style="font-weight: bold; color: ' . $bot_color . ';">' . $bot_name_ary['voila'] . '</span>';
		return $bot_name;
	}
	elseif ($tmp_list[0] == "65" && $tmp_list[1] == "19" && $tmp_list[2] == "150" && $tmp_list[3] >= 193 && $tmp_list[3] <= 256)
	{
		$bot_name = '<span style="font-weight: bold; color: ' . $bot_color . ';">' . $bot_name_ary['omniexplorer'] . '</span>';
		return $bot_name;
	}
	elseif ($tmp_list[0] == "212" && $tmp_list[1] == "27" && $tmp_list[2] == "41" && $tmp_list[3] >= 20 && $tmp_list[3] <= 50)
	{
		$bot_name = '<span style="font-weight: bold; color: ' . $bot_color . ';">' . $bot_name_ary['pompos'] . '</span>';
		return $bot_name;
	}
	elseif ( ($tmp_list[0] == "66" && $tmp_list[1] == "154" && $tmp_list[2] == "102") || ($tmp_list[0] == "66" && $tmp_list[1] == "154" && $tmp_list[2] == "103") )
	{
		$bot_name = '<span style="font-weight: bold; color: ' . $bot_color . ';">' . $bot_name_ary['gigablast'] . '</span>';
		return $bot_name;
	}
	elseif ($tmp_list[0] == "212" && $tmp_list[1] == "222" && $tmp_list[2] == "51")
	{
		$bot_name = '<span style="font-weight: bold; color: ' . $bot_color . ';">' . $bot_name_ary['ebay'] . '</span>';
		return $bot_name;
	}
	elseif ($tmp_list[0] == "65" && $tmp_list[1] == "214" && $tmp_list[2] == "44")
	{
		$bot_name = '<span style="font-weight: bold; color: ' . $bot_color . ';">' . $bot_name_ary['ask_jeeves'] . '</span>';
		return $bot_name;
	}
	elseif ($tmp_list[0] == "209" && $tmp_list[1] == "237" && $tmp_list[2] == "238")
	{
		$bot_name = '<span style="font-weight: bold; color: ' . $bot_color . ';">' . $bot_name_ary['alexa'] . '</span>';
		return $bot_name;
	}
	elseif ($tmp_list[0] == "212" && $tmp_list[1] == "48" && $tmp_list[2] == "8")
	{
		$bot_name = '<span style="font-weight: bold; color: ' . $bot_color . ';">' . $bot_name_ary['virgilio'] . '</span>';
		return $bot_name;
	}
	elseif ( ($tmp_list[0] == "66" && $tmp_list[1] == "94" && $tmp_list[2] == "229") || ($tmp_list[0] == "66" && $tmp_list[1] == "228" && $tmp_list[2] == "165") )
	{
		$bot_name = '<span style="font-weight: bold; color: ' . $bot_color . ';">' . $bot_name_ary['inktomi'] . '</span>';
		return $bot_name;
	}
	else
	{
		return $bot_name;
	}

	return $bot_name;
}

function create_server_url()
{
	// usage: $server_url = create_server_url();
	global $board_config;

	$script_name = preg_replace('/^\/?(.*?)\/?$/', '\1', trim($board_config['script_path']));
	$server_name = trim($board_config['server_name']);
	$server_protocol = ( $board_config['cookie_secure'] ) ? 'https://' : 'http://';
	$server_port = ( $board_config['server_port'] <> 80 ) ? ':' . trim($board_config['server_port']) . '/' : '/';
	$server_url = $server_protocol . $server_name . $server_port . $script_name . '/';
	$server_url = ( substr($server_url, strlen($server_url) - 2, 2) == '//' ) ? substr($server_url, 0, strlen($server_url) - 1) : $server_url;

	return $server_url;
}

// Ajaxed - BEGIN
function AJAX_headers()
{
	//No caching whatsoever
	header('Content-Type: application/xml');
	header('Expires: Thu, 15 Aug 1984 13:30:00 GMT');
	header('Last-Modified: '. gmdate('D, d M Y H:i:s') .' GMT');
	header('Cache-Control: no-cache, must-revalidate');  // HTTP/1.1
	header('Pragma: no-cache');                          // HTTP/1.0
}

function AJAX_message_die($data_ar)
{
	global $template, $db;

	if (!headers_sent())
	{
		AJAX_headers();
	}

	$template->set_filenames(array('ajax_result' => 'ajax_result.tpl'));

	foreach($data_ar as $key => $value)
	{
		if ($value !== '')
		{
			$value = utf8_encode(htmlspecialchars($value));
			// Get special characters in posts back ;)
			$value = preg_replace('#&amp;\#(\d{1,4});#i', '&#\1;', $value);

			$template->assign_block_vars('tag', array(
				'TAGNAME' => $key,
				'VALUE' => $value)
			);
		}
	}

	$template->pparse('ajax_result');

	$db->sql_close();
	exit;
}

// This function is taken from includes/bbcode.php and renamed
// We need this in special occasions
function unhtmlspecialchars($text)
{
	$text = preg_replace("/&gt;/i", ">", $text);
	$text = preg_replace("/&lt;/i", "<", $text);
	$text = preg_replace("/&quot;/i", "\"", $text);
	$text = preg_replace("/&amp;/i", "&", $text);

	return $text;
}

/**
* RFC1738 compliant replacement to PHP's rawurldecode - which actually works with unicode (using utf-8 encoding)
* @author Ronen Botzer
* @param $source [STRING]
* @return unicode safe rawurldecoded string [STRING]
* @access public
*/
function utf8_rawurldecode($source)
{
	// Strip slashes
	$source = stripslashes($source);

	$decodedStr = '';
	$pos = 0;
	$len = strlen ($source);

	while ($pos < $len)
	{
		$charAt = substr($source, $pos, 1);
		if ($charAt == '%')
		{
			$pos++;
			$charAt = substr($source, $pos, 1);
			if ($charAt == 'u')
			{
				// we got a unicode character
				$pos++;
				$unicodeHexVal = substr($source, $pos, 4);
				$unicode = hexdec($unicodeHexVal);
				$entity = "&#". $unicode .';';
				$decodedStr .= utf8_encode($entity);
				$pos += 4;
			}
			else
			{
				// we have an escaped ascii character
				$hexVal = substr ($source, $pos, 2);
				$decodedStr .= chr (hexdec ($hexVal));
				$pos += 2;
			}
		}
		else
		{
			$decodedStr .= $charAt;
			$pos++;
		}
	}

	// Add slashes before sending it back to the browser;
	// this keeps people from trying to inject SQL with some malformed string like %2527
	return addslashes($decodedStr);
}

// Used to escape AJAX data correctly.
// functions_post.php must be included before calling this function
function ajax_htmlspecialchars($text)
{
	global $html_entities_match, $html_entities_replace;

	return preg_replace($html_entities_match, $html_entities_replace, $text);
}
// Ajaxed - END

function sql_like_expression($expression)
{
	$expression = str_replace(array('_', '%'), array("\_", "\%"), $expression);
	$expression = str_replace(array(chr(0) . "\_", chr(0) . "\%"), array('_', '%'), $expression);
	if (function_exists(mysql_real_escape_string))
	{
		$like_expression = ('LIKE \'' . @mysql_real_escape_string($expression) . '\'');
	}
	else
	{
		$like_expression = ('LIKE \'' . str_replace("'", '%27', $expression) . '\'');
	}
	return $like_expression;
}


function build_topic_icon_link($forum_id, $topic_id, $topic_type, $topic_reg, $topic_replies, $topic_news_id, $topic_vote, $topic_status, $topic_moved_id, $topic_post_time, $user_replied, $replies, $unread)
{
	//build_topic_icon_link($forum_id, $topic_rowset[$i]['topic_id'], $topic_rowset[$i]['topic_type'], $topic_rowset[$i]['topic_replies'], $topic_rowset[$i]['news_id'], $topic_rowset[$i]['topic_vote'], $topic_rowset[$i]['topic_status'], $topic_rowset[$i]['topic_moved_id'], $topic_rowset[$i]['post_time'], $user_replied, $replies, $unread);
	global $board_config, $phpEx, $lang, $images, $userdata, $tracking_topics, $tracking_forums, $forum_id_append, $topic_id_append;
	global $HTTP_COOKIE_VARS;

	$topic_link = array();
	$topic_link['forum_id_append'] = $forum_id_append;
	$topic_link['topic_id_append'] = $topic_id_append;
	$topic_link['topic_id'] = $topic_id;
	$topic_link['type'] = '';
	$topic_link['icon'] = '';
	$topic_link['class'] = 'topiclink';
	$topic_link['class_new'] = '';
	$topic_link['image'] = '';
	$topic_link['image_read'] = '';
	$topic_link['image_unread'] = '';
	$topic_link['image_alt'] = '';
	$topic_link['newest_post_img'] = '';
	$upi_calc['upi_prefix'] = '';
	$upi_calc['newest_post_id'] = '';
	$icon_prefix = '';
	$icon_locked = ($topic_status == TOPIC_LOCKED) ? '_locked' : '';
	$icon_own = $user_replied ? '_own' : '';

	if( $topic_status == TOPIC_MOVED )
	{
		$topic_link['type'] = $lang['Topic_Moved'] . ' ';
		$topic_link['topic_id'] = $topic_moved_id;
		$topic_link['topic_id_append'] = POST_TOPIC_URL . '=' . $topic_moved_id;
		$topic_link['forum_id_append'] = '';
		$topic_link['image_alt'] = $lang['Topics_Moved'];
		$topic_link['newest_post_img'] = '';
		$topic_link['class'] = 'topiclink';
		$icon_prefix = 'topic_nor';
		$icon_locked = '_locked';
		$topic_link['image_read'] = $images[$icon_prefix . $icon_locked . '_read' . $icon_own];
		$topic_link['image_unread'] = $images[$icon_prefix . $icon_locked . '_unread' . $icon_own];
	}
	else
	{
		if( $topic_type == POST_GLOBAL_ANNOUNCE )
		{
			$topic_link['type'] = $lang['Topic_global_announcement'] . ' ';
			$topic_link['icon'] = '<img src="' . $images['vf_topic_ga'] . '" alt="' . $lang['Topic_global_announcement_nb'] . '" title="' . $lang['Topic_global_announcement_nb'] . '" /> ';
			$topic_link['class'] = 'topic_glo';
			$icon_prefix = 'topic_glo';
		}
		elseif( $topic_type == POST_ANNOUNCE )
		{
			$topic_link['type'] = $lang['Topic_Announcement'] . ' ';
			$topic_link['icon'] = '<img src="' . $images['vf_topic_ann'] . '" alt="' . $lang['Topic_Announcement_nb'] . '" title="' . $lang['Topic_Announcement_nb'] . '" /> ';
			$topic_link['class'] = 'topic_ann';
			$icon_prefix = 'topic_ann';
		}
		elseif( $topic_type == POST_STICKY )
		{
			$topic_link['type'] = $lang['Topic_Sticky'] . ' ';
			$topic_link['icon'] = '<img src="' . $images['vf_topic_imp'] . '" alt="' . $lang['Topic_Sticky_nb'] . '" title="' . $lang['Topic_Sticky_nb'] . '" /> ';
			$topic_link['class'] = 'topic_imp';
			$icon_prefix = 'topic_imp';
		}
		// Event Registration Icy Phoenix Edition
		elseif( $topic_reg == 1 )
		{
			$topic_link['type'] = '<img src="' . $images['vf_topic_event'] . '" alt="' . $lang['Topic_Event_nb'] . '" title="' . $lang['Topic_Event_nb'] . '" /> ' . $lang['Topic_Event'] . ' ';
			$topic_link['icon'] .= '<img src="' . $images['vf_topic_event'] . '" alt="' . $lang['Topic_Event_nb'] . '" title="' . $lang['Topic_Event_nb'] . '" /> ';
			$topic_link['class'] = 'topiclink';
			$icon_prefix = 'topic_nor';
		}
		// Event Registration Icy Phoenix Edition
		else
		{
			$topic_link['type'] = '';
			$topic_link['icon'] = '<img src="' . $images['vf_topic_nor'] . '" alt="' . $lang['Topic'] . '" title="' . $lang['Topic'] . '" /> ';
			$topic_link['class'] = 'topiclink';
			if($replies >= $board_config['hot_threshold'])
			{
				$icon_prefix = 'topic_hot';
			}
			else
			{
				$icon_prefix = 'topic_nor';
			}
		}

		$topic_link['image_read'] = $images[$icon_prefix . $icon_locked . '_read' . $icon_own];
		$topic_link['image_unread'] = $images[$icon_prefix . $icon_locked . '_unread' . $icon_own];

		if ($topic_news_id > 0)
		{
			//$topic_link['type'] = $lang['News_Cmx'] . ' ';
			$topic_link['type'] = '<img src="' . $images['vf_topic_news'] . '" alt="' . $lang['Topic_News_nb'] . '" title="' . $lang['Topic_News_nb'] . '" /> ' . $topic_link['type'];
			$topic_link['icon'] = $topic_link['icon'] . '<img src="' . $images['vf_topic_news'] . '" alt="' . $lang['Topic_News_nb'] . '" title="' . $lang['Topic_News_nb'] . '" /> ';
		}

		if( $topic_vote )
		{
			//$topic_link['type'] .= $lang['Topic_Poll'] . ' ';
			$topic_link['type'] .= '<img src="' . $images['vf_topic_poll'] . '" alt="' . $lang['Topic_Poll_nb'] . '" title="' . $lang['Topic_Poll_nb'] . '" /> ';
			$topic_link['icon'] .= '<img src="' . $images['vf_topic_poll'] . '" alt="' . $lang['Topic_Poll_nb'] . '" title="' . $lang['Topic_Poll_nb'] . '" /> ';
		}
	}

	if( $userdata['session_logged_in'] )
	{
		//-----------------------------------------------------------
		//<!-- BEGIN Unread Post Information to Database Mod -->
		if( !$userdata['upi2db_access'] || !is_array($unread) )
		{
		//<!-- END Unread Post Information to Database Mod -->
		//------------------------------------------------------------

			if( $topic_post_time > $userdata['user_lastvisit'] )
			{
				if( !empty($tracking_topics) || !empty($tracking_forums) || isset($HTTP_COOKIE_VARS[$board_config['cookie_name'] . '_f_all']) )
				{
					$unread_topics = true;

					if( !empty($tracking_topics[$topic_link['topic_id']]) )
					{
						if( $tracking_topics[$topic_link['topic_id']] >= $topic_post_time )
						{
							$unread_topics = false;
						}
					}

					if( !empty($tracking_forums[$forum_id]) )
					{
						if( $tracking_forums[$forum_id] >= $topic_post_time )
						{
							$unread_topics = false;
						}
					}

					if( isset($HTTP_COOKIE_VARS[$board_config['cookie_name'] . '_f_all']) )
					{
						if( $HTTP_COOKIE_VARS[$board_config['cookie_name'] . '_f_all'] >= $topic_post_time )
						{
							$unread_topics = false;
						}
					}
				}
				else
				{
					$unread_topics = true;
				}
			}
			else
			{
				$unread_topics = false;
			}
		//--------------------------------------------------------
		//<!-- BEGIN Unread Post Information to Database Mod -->
		}
		else
		{
			$upi_calc = upi_calc_unread_simple($unread, $topic_link['topic_id']);
			$unread_topics = $upi_calc['unread'];
			//$topic_link['type'] = $upi_calc['upi_prefix'] . $topic_link['type'];
			$upi_calc['newest_post_id'] = $post_id;
		}
		//<!-- END Unread Post Information to Database Mod -->
		//--------------------------------------------------------
	}
	else
	{
		$unread_topics = false;
	}

	if($unread_topics == true)
	{
		$topic_link['class_new'] = '-new';
		$topic_link['image'] = $topic_link['image_unread'];
		$topic_link['image_alt'] = ( $topic_status == TOPIC_LOCKED ) ? $lang['Topic_locked'] : $lang['New_posts'];
		$topic_link['newest_post_img'] = '';
		if (empty($upi_calc['newest_post_id']))
		{
			$newest_post_img_url = append_sid(VIEWTOPIC_MG . '?' . $topic_link['forum_id_append'] . '&amp;' . $topic_link['topic_id_append'] . '&amp;view=newest');
		}
		else
		{
			$newest_post_img_url = append_sid(VIEWTOPIC_MG . '?' . $topic_link['forum_id_append'] . '&amp;' . $topic_link['topic_id_append'] . '&amp;' . POST_POST_URL . '=' . $upi_calc['newest_post_id']) . '#p' . $upi_calc['newest_post_id'];
		}
		$topic_link['newest_post_img'] = '<a href="' . $newest_post_img_url . '"><img src="' . $images['icon_newest_reply'] . '" alt="' . $lang['View_newest_post'] . '" title="' . $lang['View_newest_post'] . '" /></a> ' . $upi_calc['upi_prefix'];
	}
	else
	{
		$topic_link['class_new'] = '';
		$topic_link['image'] = $topic_link['image_read'];
		$topic_link['image_alt'] = ( $topic_status == TOPIC_LOCKED ) ? $lang['Topic_locked'] : $lang['No_new_posts'];
		$topic_link['newest_post_img'] = '';
	}
	return $topic_link;
}

function upi_calc_unread_simple($unread, $topic_id)
{
	global $userdata, $lang;

	$upi2db_status = '';
	if (in_array($topic_id, $unread['new_topics']) || in_array($topic_id, $unread['edit_topics']))
	{
		if((in_array($topic_id, $unread['new_topics']) && in_array($topic_id, $unread['edit_topics'])) && $userdata['user_upi2db_new_word'] && $userdata['user_upi2db_edit_word'])
		{
			$upi2db_status = $lang['upi2db_post_edit'] . $lang['upi2db_post_and'] . $lang['upi2db_post_new'] . ': ';
		}
		else
		{
			if(in_array($topic_id, $unread['new_topics']) && $userdata['user_upi2db_new_word'])
			{
				$upi2db_status = $lang['upi2db_post_new'] . ': ';
			}

			if(in_array($topic_id, $unread['edit_topics']) && $userdata['user_upi2db_edit_word'])
			{
				$upi2db_status = $lang['upi2db_post_edit'] . ': ';
			}
		}
		$min_new_post_id = (empty($unread[$topic_id]['new_posts'])) ? '99999999' : min($unread[$topic_id]['new_posts']);
		$min_edit_post_id = (empty($unread[$topic_id]['edit_posts'])) ? '99999999' :  min($unread[$topic_id]['edit_posts']);
		$post_id = ($min_edit_post_id >= $min_new_post_id) ? $min_new_post_id : $min_edit_post_id;
		$upi_calc['unread'] = true;
		$upi_calc['upi_prefix'] = $upi2db_status;
		$upi_calc['newest_post_id'] = $post_id;
	}
	else
	{
		$upi_calc['unread'] = false;
		$upi_calc['upi_prefix'] = '';
		$upi_calc['newest_post_id'] = '';
	}
	return $upi_calc;
}

function user_replied_array($topic_rowset)
{
	global $userdata, $db, $board_config;
	$user_topics = array();
	if ( ($userdata['user_id'] != ANONYMOUS) && ($board_config['enable_own_icons'] == true) )
	{
		// get all the topic ids to display
		$topic_ids = array();
		for ($i = 0; $i < count($topic_rowset); $i++)
		{
			$topic_ids[] = intval(substr($topic_rowset[$i]['topic_id'], 0));
		}
		// check if the user replied to
		if (!empty($topic_ids))
		{
			// check the posts
			$s_topic_ids = implode(', ', $topic_ids);
			$sql = "SELECT DISTINCT topic_id FROM " . POSTS_TABLE . "
					WHERE topic_id IN ($s_topic_ids)
						AND poster_id = '" . $userdata['user_id'] . "'";
			if ( !($result = $db->sql_query($sql)) )
			{
				message_die(GENERAL_ERROR, 'Could not obtain post information', '', __LINE__, __FILE__, $sql);
			}
			while ($row = $db->sql_fetchrow($result))
			{
				$user_topics[$row['topic_id']] = true;
			}
			$db->sql_freeresult($result);
		}
	}
	return $user_topics;
}

function user_check_friend($target_id)
{
	global $userdata, $db;
	$sql = "SELECT * FROM " . ZEBRA_TABLE . "
			WHERE user_id = '" . $userdata['user_id'] . "'
				AND zebra_id = '" . $target_id . "'
				AND friend = '1'";
	if ( !($result = $db->sql_query($sql)) )
	{
		message_die(GENERAL_ERROR, 'Could not obtain friends information', '', __LINE__, __FILE__, $sql);
	}
	while ($row = $db->sql_fetchrow($result))
	{
		$db->sql_freeresult($result);
		return true;
	}
	return false;
}

function user_get_zebra_list($ftype = 'friends')
{
	global $userdata, $db;
	if ($ftype == 'foes')
	{
		$sql_f_check = 'foe';
	}
	else
	{
		$sql_f_check = 'friend';
	}
	$zebra_list = array();
	$sql = "SELECT z.zebra_id, u.username
			FROM " . ZEBRA_TABLE . " z, " . USERS_TABLE . " u
			WHERE z.user_id = '" . $userdata['user_id'] . "'
				AND " . $sql_f_check . " = '1'
				AND u.user_id = z.zebra_id
			ORDER BY u.username ASC";
	if ( !($result = $db->sql_query($sql, false, 'zebra_users_')) )
	{
		message_die(GENERAL_ERROR, 'Could not obtain friends information', '', __LINE__, __FILE__, $sql);
	}
	while ($row = $db->sql_fetchrow($result))
	{
		$zebra_list[] = $row['zebra_id'];
	}
	$db->sql_freeresult($result);
	if (empty($zebra_list))
	{
		return false;
	}
	else
	{
		return $zebra_list;
	}
}

function user_get_friends_online_list()
{
	global $userdata, $db;
	$friends_online_list = array();
	$sql = "SELECT u.user_id, u.username, u.user_allow_viewonline, s.session_logged_in, s.session_time
					FROM " . ZEBRA_TABLE . " z, " . USERS_TABLE . " u, " . SESSIONS_TABLE . " s
					WHERE z.user_id = '" . $userdata['user_id'] . "'
					AND z.friend = '1'
					AND u.user_id = z.zebra_id
					AND u.user_id = s.session_user_id
					AND s.session_time >= " . ( time() - ONLINE_REFRESH ) . "";
	if ( !($result = $db->sql_query($sql)) )
	{
		message_die(GENERAL_ERROR, 'Could not obtain friends information', '', __LINE__, __FILE__, $sql);
	}
	while ($row = $db->sql_fetchrow($result))
	{
		$friends_online_list[$row['user_id']]['username'] = $row['username'];
		$friends_online_list[$row['user_id']]['user_level'] = $row['user_level'];
		$friends_online_list[$row['user_id']]['user_allow_viewonline'] = $row['user_allow_viewonline'];
	}
	$db->sql_freeresult($result);
	if (empty($friends_online_list))
	{
		return false;
	}
	else
	{
		return $friends_online_list;
	}
}

function user_check_admin_mod($target_id)
{
	global $db;
	$sql = "SELECT * FROM " . USERS_TABLE . "
			WHERE user_id = '" . $target_id . "'
				AND user_level > 0";
	if ( !($result = $db->sql_query($sql, false, 'user_level_')) )
	{
		message_die(GENERAL_ERROR, 'Could not obtain friends information', '', __LINE__, __FILE__, $sql);
	}
	while ($row = $db->sql_fetchrow($result))
	{
		$db->sql_freeresult($result);
		return true;
	}
	return false;
}

function user_check_pm_in_allowed($target_id)
{
	global $userdata, $db;
	$sql = "SELECT * FROM " . USERS_TABLE . "
			WHERE user_id = '" . $target_id . "'
				AND user_allow_pm_in = 1";
	if ( !($result = $db->sql_query($sql, false, 'user_pm_')) )
	{
		message_die(GENERAL_ERROR, 'Could not obtain friends information', '', __LINE__, __FILE__, $sql);
	}
	while ($row = $db->sql_fetchrow($result))
	{
		$db->sql_freeresult($result);
		return true;
	}
	return false;
}

function get_default_avatar($user_id, $path_prefix = '')
{
	global $board_config;

	$avatar_img = '&nbsp;';
	if ($board_config['default_avatar_set'] != 3)
	{
		if (($board_config['default_avatar_set'] == 0) && ($user_id == ANONYMOUS) && ($board_config['default_avatar_guests_url'] != ''))
		{
			$avatar_img = '<img src="' . $path_prefix . $board_config['default_avatar_guests_url'] . '" alt="" />';
		}
		elseif (($board_config['default_avatar_set'] == 1) && ($user_id != ANONYMOUS) && ($board_config['default_avatar_users_url'] != '') )
		{
			$avatar_img = '<img src="' . $path_prefix . $board_config['default_avatar_users_url'] . '" alt="" />';
		}
		elseif ($board_config['default_avatar_set'] == 2)
		{
			if (($user_id == ANONYMOUS) && ($board_config['default_avatar_guests_url'] != ''))
			{
				$avatar_img = '<img src="' . $path_prefix . $board_config['default_avatar_guests_url'] . '" alt="" />';
			}
			elseif (($user_id != ANONYMOUS) && ($board_config['default_avatar_users_url'] != ''))
			{
				$avatar_img = '<img src="' . $path_prefix . $board_config['default_avatar_users_url'] . '" alt="" />';
			}
		}
	}

	return $avatar_img;
}

function user_get_avatar($user_id, $user_avatar, $user_avatar_type, $user_allow_avatar, $path_prefix = '')
{
	global $board_config;
	$user_avatar_link = '';
	if ($user_avatar_type && ($user_id != ANONYMOUS) && $user_allow_avatar)
	{
		switch($user_avatar_type)
		{
			case USER_AVATAR_UPLOAD:
				$user_avatar_link = ($board_config['allow_avatar_upload']) ? '<img src="' . $path_prefix . $board_config['avatar_path'] . '/' . $user_avatar . '" alt="" />' : '';
				break;
			case USER_AVATAR_REMOTE:
				$user_avatar_link = resize_avatar($user_avatar);
				break;
			case USER_AVATAR_GALLERY:
				$user_avatar_link = ($board_config['allow_avatar_local']) ? '<img src="' . $path_prefix . $board_config['avatar_gallery_path'] . '/' . $user_avatar . '" alt="" />' : '';
				break;
			case USER_GRAVATAR:
				$user_avatar_link = ($board_config['enable_gravatars']) ? '<img src="' . get_gravatar($user_avatar) . '" alt="" />' : '';
				break;
			default:
				$user_avatar_link = '';
		}
	}

	if ($user_avatar_link == '')
	{
		$user_avatar_link = get_default_avatar($user_id, $path_prefix);
	}

	return $user_avatar_link;
}

/*
* $topic_prefixes = get_topic_prefixes();
*/
function get_topic_prefixes()
{
	global $db;
	$sql = "SELECT * FROM " . TITLE_INFOS_TABLE . " ORDER BY title_info ASC";
	if ( !($result = $db->sql_query($sql, false, 'topics_prefixes_')) )
	{
		message_die(GENERAL_MESSAGE, 'Unable to query Quick Title Addon informations.');
	}
	$topic_prefixes = array();
	while ($row = $db->sql_fetchrow($result))
	{
		$topic_prefixes[$row['id']] = $row['title_info'];
	}
	$db->sql_freeresult($result);
	return $topic_prefixes;
}

function user_get_thanks_received($user_id)
{
	global $db;
	$total_thanks_received = 0;
	$sql = "SELECT COUNT(th.topic_id) AS total_thanks
					FROM " . THANKS_TABLE . " th, " . TOPICS_TABLE . " t
					WHERE t.topic_poster = '" . $user_id . "'
						AND t.topic_id = th.topic_id";
	if(!$result = $db->sql_query($sql))
	{
		message_die(GENERAL_ERROR, "Could not query thanks informations", "", __LINE__, __FILE__, $sql);
	}
	if (!$row = $db->sql_fetchrow($result))
	{
		message_die(GENERAL_ERROR, "Could not query thanks informations", "", __LINE__, __FILE__, $sql);
	}
	$total_thanks_received = $row['total_thanks'];
	$db->sql_freeresult($result);
	return $total_thanks_received;
}

function empty_cache_folder($phpbb_root_path = './', $cg = false)
{
	global $phpEx;

	$skip_files = array(
		'.',
		'..',
		'.htaccess',
		'index.htm',
		'index.html',
		'index.' . $phpEx,
		'attach_config.' . $phpEx,
	);

	$sql_prefix = 'sql_';
	$tpl_prefix = 'tpl_';
	$phpbb_update_prefix = 'phpbb_';
	$cache_prefix = 'cache_';
	$cg_prefix = 'u_';
	$dat_extension = '.dat';

	$dir = $phpbb_root_path . MAIN_CACHE_FOLDER;
	$res = @opendir($dir);
	while( ($file = readdir($res)) !== false )
	{
		if ($cg == true)
		{
			if ( preg_match('/' . $cg_prefix . '/', $file) && !in_array($file, $skip_files) )
			{
				$res2 = @unlink($dir . $file);
			}
		}
		else
		{
			if ( !in_array($file, $skip_files) )
			{
				$res2 = @unlink($dir . $file);
			}
		}
	}
	closedir($res);
	return true;
}

// Activity - BEGIN
//if ( defined('ACTIVITY_MOD') )
if ( defined('ACTIVITY_MOD') && (ACTIVITY_MOD == true) )
{
	include_once( $phpbb_root_path . ACTIVITY_MOD_PATH . 'includes/functions_amod_includes_functions.' . $phpEx );
}
// Activity - END

function check_medal_mod($medal_id)
{
	global $db, $userdata;
	
	$sql = "SELECT *
	FROM " . MEDAL_MOD_TABLE . "  
	WHERE medal_id =" . $medal_id;
	
	if ( !($result = $db->sql_query($sql)) )
	{
		message_die(GENERAL_ERROR, 'Could not obtain user and medal information', '', __LINE__, __FILE__, $sql);
	}

	$medal_info = array();
	$found = FALSE;
	while ( $medal_info = $db->sql_fetchrow($result) )
	{

		$medal_moderator = $medal_info['user_id'];

		if ( $medal_moderator == $userdata['user_id'] )
		{
			$found = TRUE;
		}
	}
	$db->sql_freeresult($result);
	
	return $found;
}

?>